/**************************************************************************
 * Name:      WdgCommProtocol.h
 * Purpose:   Communication protocol definition
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 *
 **************************************************************************/

#if !defined(__WdgCommProtocol_h)
#define __WdgCommProtocol_h

#ifdef __WdgCommProtocol_c
#define GLOBAL
#else
#define GLOBAL extern
#endif

/**************************************************************************
* definitions
**************************************************************************/

#define UART_SYNC_BYTE      0xAA


typedef struct {
  u08 bSec;
  u08 bMin;
  u08 bHour;

  u08 bDay;
  u08 bMonth;
  u08 bYear;      // + 2000

} TWDGDateTime;

typedef enum {
  WD_CMD_TRIGGER,
  WD_CMD_INFO,
  WD_CMD_SETINGS,
  WD_CMD_LOG,
  WD_CMD_CLEAR_LOG,

  WD_CMD_MAX_CMDS
} TCommCmds;

typedef enum {
  WD_CMD_GET_SETTINGS,
  WD_CMD_SET_SETTINGS,
  WD_CMD_MAX_SETT_CMDS
} TCommSettingsCmds;



typedef struct {
  u16convert  SWVersion;                // Software Version
  u32 dwResetTime;                      // time to reset in seconds
  u32 dwRunTime;                        // Runtime in seconds
  u32 dwResetsCount;                    // count of resets
  u32 dwTriggerCount;                   // count of triggers
  u32 dwLastTriggerTime;                // time since last trigger
  u08 bBeeping;                         // Beeper is used
  u08 bStopReset;                       // Reset or not reset
  u32 dwLogEntryCount;                  // count of log entries

} PACKED TWD_ComInfoData;

#define WDG_SET_MASK_RESET_TIME         0x00000001
#define WDG_SET_MASK_BEEP               0x00000002
#define WDG_SET_MASK_STOP_RESET         0x00000004
#define WDG_SET_MASK_RESET_IT           0x00000008
#define WDG_SET_MASK_CLEAR_CNT          0x00000010

typedef struct {
  u08 bSettingsCmd;
  u32 dwSettings;                       // Bit field what to set

  u32 dwResetTime;                      // Time to wait till reset
  u08 bBeep;                            // set Beeper on or off
  u08 bStopReset;                       // Stop or Start to reset
  u08 bResetIt;                         // Do Reset
  u08 bClearCounters;                   // Clear all counters

} PACKED TWD_ComSettingsData;


typedef struct {
  u32 dwID;                          // 4  // 12
  u32 dwThisResetCounter;            // 4  // 16
  u32 dwRunTimeAtReset;              // 4  // 20
  TWDGDateTime DateTimeAtReset;      // 6  // 24
} TWDLogItem;


typedef struct {
  u32       dwIndex;                 // 4  // 8
  TWDLogItem  LogItem;               // 18 // 12

} PACKED TWD_LogEntry;

typedef struct {
 u08 bSyncByte;                       // 1  // 0
 u08 bCmd;                            // 1  // 1

 TWDGDateTime WDGDateTime;            // 6  // 2

  union {
    TWD_ComInfoData      Info;
    TWD_ComSettingsData  Settings;
    TWD_LogEntry         LogEntry;
  } Data;                                   // 8

} PACKED TWD_CommData;





/**************************************************************************
* declarations
**************************************************************************/



#undef GLOBAL
#endif /* __WdgCommProtocol_h  */
/*********************************** EOF *********************************/
