/**************************************************************************
*
*  Description:   CRC Calculation procedures
 * Name:          fcs.c
 * Purpose:
 * Author:        MasterAlexei (MasterAlexei@gmail.com)
 * Created:
 * Copyright:     MasterAlexei (http://www.fun-electronic.net/)
 *
 * License:
 *
 *  Copyright (c) 2005-2009 by Govorukhin Alexey
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 ****************************************************************************/
#if !defined(__FCS_C__)
#define __FCS_C__

#include "types.h"
#include "fcs.h"

#ifndef __WXMSW__
#ifndef __WXGTK__

#ifdef FCS_COMMUNICATION

#include "DateTime.h"

#endif

#endif
#endif

#if defined(FCS_COMMUNICATION) || defined(__AVR__)|| defined(__C51__)|| defined(SDCC)

#include "rf_protdefs.h"

#endif

/**************************************************************************
* definitions
**************************************************************************/

#define INITIALCRCVALUE (unsigned short)0xFFFF

/**************************************************************************
* declarations
**************************************************************************/


/**************************************************************************
* function prototypes
**************************************************************************/
u16 CrcLookupTableIndex(register u08 bIndex);

/**************************************************************************
* implementation
**************************************************************************/

u16 CalculateCRC(register u08* pDatenbyte,
                  register u16  wCnt)
{
   register u16 wNewCrc = INITIALCRCVALUE;

   while (wCnt--)
   #ifdef __CC1010
     wNewCrc = (wNewCrc >> 8) ^ CodeMem.crcLookupTable[(wNewCrc ^ *pDatenbyte++) & 0xff];
   #else
     wNewCrc = (wNewCrc >> 8) ^ CrcLookupTableIndex((wNewCrc ^ *pDatenbyte++) & 0xff);
   #endif

   return (wNewCrc ^ 0xFFFF);
}

u16 CalculateCRCI(register u08* pDatenbyte,
                  register u16  wCnt, u16 wInitVal )
{
   register u16 wNewCrc = wInitVal;

   while (wCnt--)
   #ifdef __CC1010
     wNewCrc = (wNewCrc >> 8) ^ CodeMem.crcLookupTable[(wNewCrc ^ *pDatenbyte++) & 0xff];
   #else
     wNewCrc = (wNewCrc >> 8) ^ CrcLookupTableIndex((wNewCrc ^ *pDatenbyte++) & 0xff);
   #endif

   return (wNewCrc ^ 0xFFFF);
}

u16 CrcLookupTableIndex(register u08 bIndex)
{
   register u16 wValue;
   register u16 wShift;
   wValue = (u16)bIndex;

   for(wShift = 8; wShift--; )
      wValue = ((wValue & 1) ? ((wValue >> 1) ^ 0x8408) : (wValue >> 1));

   return wValue;
}

void CopyBack(u08 * bBuffer, u16 wLen)
{
  u16 i;
  for (i = wLen; i > 0; i--)
  {
    bBuffer[i] = bBuffer[i - 1];
  }
}

// #ifndef __WXMSW__
#if defined(FCS_COMMUNICATION) || defined(__AVR__) || defined(__C51__)|| defined(SDCC)

u16 SetStartStopsBytes(u08 * bBuffer, u16 wLen)
{
 u16 wNewLen = wLen;

 CopyBack(bBuffer, wNewLen);
 wNewLen++;
 bBuffer[0]         = COMM_SYNC_BYTE;
 bBuffer[wNewLen++] = COMM_SYNC_BYTE;

 return wNewLen;
}

u16 PreparePacketToTransfer(u08 * bBuffer, u16 wLen)
{
  u16 i;
  u16 wNewLen = wLen;
  for (i = 0; i < wNewLen; i++)
  {
    if (bBuffer[i] == COMM_SYNC_BYTE || bBuffer[i] == COMM_ESC_BYTE)
    {
      CopyBack(&bBuffer[i], wNewLen - i);
      bBuffer[i]     = COMM_ESC_BYTE;
      i += 1;
      bBuffer[i] ^= 0x20;
      wNewLen++;
    }
  }
  return(wNewLen);
}
#endif
// #endif

#endif // __FCS_C__
/*********************************** EOF *********************************/
