/***************************************************************
 * Name:      tasks.c
 * Purpose:
 * Author:    MasterAlexei (MasterAlexei@gmail.com)
 * Created:
 * Copyright: MasterAlexei (http://www.fun-electronic.net/)
 *
 * License:
 *
 *  Copyright (c) 2005-2009 by Govorukhin Alexey
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 ****************************************************************************/

#if !defined(__tasks_c)
#define __tasks_c

/*
#if !defined(__WXMSW__) && !defined(__WXGTK__) && !defined(_AVRProgrammer_)
#include "incfiles.h"
#else
*/
// #include "struct.h"
#include <string.h>
#include "types.h"
#include "TasksDefs.h"
#include "tasks.h"
//#endif


/**************************************************************************
* definitions
**************************************************************************/
/*
#ifndef _UNILINK_
#error _UNILINK_ not defined
#endif
*/
/**************************************************************************
* declarations
**************************************************************************/

u08 volatile bFirstTask[TSK_MAX_TASKS];
u08 volatile bLastTask[TSK_MAX_TASKS];
TTaskElement volatile TasksQueue[TSK_MAX_TASKS * MAX_TOTAL_TASKS_IN_QUEUE];

static u08 bSemafor = 0;

/**************************************************************************
* implementation
**************************************************************************/
void InitTasks(void)
{
   memset((u08 *)bFirstTask, 0, TSK_MAX_TASKS);
   memset((u08 *)bLastTask,  0, TSK_MAX_TASKS);
   memset((u08 *)TasksQueue, 0, MAX_TOTAL_TASKS_IN_QUEUE * TSK_MAX_TASKS);
}

/**************************************************************************
* implementation
**************************************************************************/


void ClearAllTasks(u08 TaskQueueNumber)
{
  if(TaskQueueNumber < TSK_MAX_TASKS)
  {
    bLastTask[TaskQueueNumber] = 0;
    bFirstTask[TaskQueueNumber] = 0;
  }
}

u08 GetFreeTaskPlace(u08 TaskQueueNumber)
{
 u08 ReturnVal;
   if(TaskQueueNumber < TSK_MAX_TASKS)
   {
     if(bFirstTask[TaskQueueNumber] == bLastTask[TaskQueueNumber])
       ReturnVal = MAX_TOTAL_TASKS_IN_QUEUE - 1;
     else if(bFirstTask[TaskQueueNumber] < bLastTask[TaskQueueNumber])
     {
       ReturnVal = ( (MAX_TOTAL_TASKS_IN_QUEUE - 1) - ( bLastTask[TaskQueueNumber] - bFirstTask[TaskQueueNumber] ) );
     }
     else
       ReturnVal = ( bFirstTask[TaskQueueNumber] - bLastTask[TaskQueueNumber]  - 1);
   }

   else
     ReturnVal = 0;

 return(ReturnVal);
}


// Returned next Task in TaskQueueNumber TaskQueue
u08 GetNextTask(u08 TaskQueueNumber)
{
 u08 RetVal = 0;
   if(TaskQueueNumber < TSK_MAX_TASKS)
   {
     if(GetFreeTaskPlace(TaskQueueNumber) < (MAX_TOTAL_TASKS_IN_QUEUE - 1))
     {
       RetVal = TasksQueue[TaskQueueNumber * MAX_TOTAL_TASKS_IN_QUEUE + bFirstTask[TaskQueueNumber]].bTask;
     }
   }
 return(RetVal);
}

// Add Task in TaskQueueNumber TaskQueue
u08 AddTask(u08 TaskQueueNumber, u08 TaskID)
{
 u08 fProcessed = 0;
 #ifdef _TASK_SEMAFORING_
 if( !bSemafor )
 #endif
 {
   bSemafor = 1;
   if(TaskQueueNumber < TSK_MAX_TASKS)
   {
     if(GetFreeTaskPlace(TaskQueueNumber) != 0)
     {
      TasksQueue[TaskQueueNumber * MAX_TOTAL_TASKS_IN_QUEUE + bLastTask[TaskQueueNumber]].bTask = TaskID;
      bLastTask[TaskQueueNumber]++;
      if(bLastTask[TaskQueueNumber] >= MAX_TOTAL_TASKS_IN_QUEUE)
       bLastTask[TaskQueueNumber] = 0;

      fProcessed = 1;
     }
   }
   bSemafor = 0;
 }
 return(fProcessed);
}

// Erased Task from Queue
void DoneTask(u08 TaskQueueNumber)
{
   bSemafor = 1;
   if(TaskQueueNumber < TSK_MAX_TASKS)
   {
     if(GetFreeTaskPlace(TaskQueueNumber) < (MAX_TOTAL_TASKS_IN_QUEUE - 1))
     {
      bFirstTask[TaskQueueNumber]++;
      if(bFirstTask[TaskQueueNumber] >= MAX_TOTAL_TASKS_IN_QUEUE)
       bFirstTask[TaskQueueNumber] = 0;
     }
   }
   bSemafor = 0;
}




#endif /* __tasks_c  */
/*********************************** EOF *********************************/
