/***************************************************************
 * Name:      Config.cpp
 * Purpose:   Configuration class
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/
#if !defined(__CONFIG_CPP_)
#define __CONFIG_CPP_

#include <stdio.h>
#include <string.h>

#include "struct.h"

#include "Config.h"

#include "Logging.h"


/**************************************************************************
* definitions
**************************************************************************/


/**************************************************************************
* declarations
**************************************************************************/

TConfig * pGlobalConfig = NULL;

/**************************************************************************
* function prototypes
**************************************************************************/


/**************************************************************************
* implementation
**************************************************************************/

TConfig::TConfig(char * szConfigFileName)
{
  szConfigFile = szConfigFileName;
  fReady = false;


  if( Load() )
  {
    fReady = true;
  }
}

TConfig::~TConfig()
{

}

#define CFGG_WATCHDOG (char *)"WatchDog"
#define CFGG_NETWORK  (char *)"Network"
#define CFGG_LOGGING  (char *)"Logging"

#define CFG_DEFAULT_BAUDRATE    38400L
#define CFG_DEFAULT_LOCAL_PORT  10000
#define CFG_DEFAULT_LOG_LEVEL   1
#define CFG_DEFAULT_TRIGGER_DELAY 10000
#define CFG_DEFAULT_ANSWER_WAIT_DELAY 1000
#define CFG_DEFAULT_LOG_TO_CONSOLE 1

bool TConfig::Load()
{
  bool fRes = false;

  GKeyFile *ConfigFile;
  GKeyFileFlags gFlags;
  GError *gError = NULL;


  ConfigFile = g_key_file_new ();
  gFlags = (GKeyFileFlags)( G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS );

  if ( g_key_file_load_from_file ( ConfigFile , szConfigFile, gFlags, &gError ) )
  {
    CfgGetString( ConfigFile, CFGG_WATCHDOG, (char *)"ComPortName", (char *)Config.szComPort, CFG_COMPORT_NAME_LEN, (char *)"/dev/ttyS0" );
    CfgGetString( ConfigFile, CFGG_LOGGING, (char *)"LogFile", (char *)Config.szLogFileName, CFG_LOG_FILE_NAME_LEN, (char *)"/var/log/wdtlog.log" );

    Config.dwBautRate     = CfgGetInteger(ConfigFile, CFGG_WATCHDOG, (char *)"BaudRate", CFG_DEFAULT_BAUDRATE );
    Config.dwLocalPort    = CfgGetInteger(ConfigFile, CFGG_NETWORK,  (char *)"LocalPort", CFG_DEFAULT_LOCAL_PORT );
    Config.bLogLevel      = CfgGetInteger(ConfigFile, CFGG_LOGGING,  (char *)"LogLevel", CFG_DEFAULT_LOG_LEVEL );
    Config.dwTriggerDelay = CfgGetInteger(ConfigFile, CFGG_WATCHDOG, (char *)"TriggerDelay", CFG_DEFAULT_TRIGGER_DELAY );
    Config.dwAnswerDelay  = CfgGetInteger(ConfigFile, CFGG_WATCHDOG, (char *)"AnswerWaitDelay", CFG_DEFAULT_ANSWER_WAIT_DELAY );

    Config.bLogToConsole  = CfgGetInteger(ConfigFile, CFGG_LOGGING,  (char *)"LogToConsole", CFG_DEFAULT_LOG_TO_CONSOLE );


    LOG(LOG_INFO, "Loaded Config:");
    LOG(LOG_INFO, "\tBaudRate:   \t%lu\n"
            "\tComPortName:\t%s\n"
            "\tLocalPort:  \t%lu\n"
            "\tLogFile:    \t%s\n"
            "\tLogLevel:   \t%u\n"
            "\tTrigger Delay:\t%lu",
            Config.dwBautRate,
            Config.szComPort,
            Config.dwLocalPort,
            Config.szLogFileName,
            Config.bLogLevel,
            Config.dwTriggerDelay);

    fRes = true;
    g_key_file_free( ConfigFile );
  }
  else
  {
    LOG(LOG_INFO, "PCWatchDog: Error Creating ConfigFile Object");
    LOG(LOG_ERROR, "%s", gError->message );
  }
  return fRes;
}

int TConfig::CfgGetInteger(GKeyFile * pKeyFile, char * szGroupName, char * szKey, int nDefaultValue )
{
  int nRet = 0;
  GError *g_error;
  nRet = g_key_file_get_integer( pKeyFile, szGroupName, szKey, &g_error );

  if( nRet == 0 )
  {
    #undef code
    if( ( g_error->code == G_KEY_FILE_ERROR_KEY_NOT_FOUND ) || ( g_error->code == G_KEY_FILE_ERROR_INVALID_VALUE ) )
    {
      LOG(LOG_ERROR, "Error reading integer value %s in group %s", szKey, szGroupName);
      nRet = nDefaultValue;
    }
  }
  return nRet;
}

bool TConfig::CfgGetString(GKeyFile * pKeyFile, char * szGroupName, char * szKey, char * szDstStr, int nLen, char * szDefaultValue )
{
  bool fRet = false;
  GError *error;
  char * szRes;

  szRes = g_key_file_get_string( pKeyFile, szGroupName, szKey, &error );

  if(szRes != NULL )
  {
    int nResLen = strlen( szRes );
    if( nResLen >= nLen )
    {
      nResLen = nLen - 1;
    }

    if( szDstStr != NULL )
    {
      memcpy( szDstStr, szRes, nLen );
      szDstStr[ nLen ] = 0;
      fRet = true;
    }
    delete szRes;
  }
  else
  {
    int nResLen = strlen( szDefaultValue );
    if( nResLen >= nLen )
    {
      nResLen = nLen - 1;
    }
    memcpy( szDstStr, szDefaultValue, nLen );
    szDstStr[ nLen ] = 0;
  }

  return fRet;
}

#endif /* _CONFIG_CPP_ */
/*********************************** EOF *********************************/
