/***************************************************************
 * Name:      Config.h
 * Purpose:   Configuration class
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/
#if !defined(CONFIG_H_INCLUDED)
#define CONFIG_H_INCLUDED

#if defined(__CONFIG_CPP_)
#define GLOBAL
#else
#define GLOBAL extern
#endif

#undef code
#include <glib.h>

#include "types.h"

/**************************************************************************
* definitions
**************************************************************************/
#define CFG_COMPORT_NAME_LEN  20
#define CFG_LOG_FILE_NAME_LEN  256

typedef struct TConfigData {

 char szComPort[ CFG_COMPORT_NAME_LEN ];
 u32 dwBautRate;
 u32 dwTriggerDelay;
 u32 dwAnswerDelay;

 u32 dwLocalPort;

 char szLogFileName[CFG_LOG_FILE_NAME_LEN];
 u08  bLogLevel;
 u08  bLogToConsole;

} TConfigData;

class TConfig {

private:
  char * szConfigFile;

  TConfigData Config;

  bool fReady;

  bool Load();
  int  CfgGetInteger(GKeyFile * pKeyFile, char * szGroupName, char * szKey, int nDefaultValue );
  bool CfgGetString(GKeyFile * pKeyFile, char * szGroupName, char * szKey, char * szDstStr, int nLen, char * szDefaultValue );

public:
  TConfig(char * szConfigFileName);
  ~TConfig();

  u32 GetLocalPort() {return Config.dwLocalPort;}
  u08 GetLogLevel() { return Config.bLogLevel;}

  char * GetComPortName() { return Config.szComPort; }

  u32 GetTriggerDelay() {return Config.dwTriggerDelay; }
  void SetTriggerDelay( u32 dwVal ) { Config.dwTriggerDelay = dwVal; }

  u32 GetAnswerDelay() { return Config.dwAnswerDelay; }
  void SetAnswerDelay( u32 dwVal ) { Config.dwAnswerDelay = dwVal; }

  char * GetLogFileName() { return Config.szLogFileName; }
  bool   LogToConsole() { return (Config.bLogToConsole == 1);}

};
/**************************************************************************
* declarations
**************************************************************************/


/**************************************************************************
* function prototypes
**************************************************************************/


/**************************************************************************
* implementation
**************************************************************************/




#undef GLOBAL
#endif /* */
/*********************************** EOF *********************************/
