/***************************************************************
 * Name:      GlobalData.h
 * Purpose:   Global Data class
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/
#ifndef _GLOBALDATA_H
#define _GLOBALDATA_H

#if defined( _GLOBALDATA_C )
#define GLOBAL
#else
#define GLOBAL extern
#endif



/**************************************************************************
* definitions
**************************************************************************/

typedef struct {
  u16convert  WDGSWVersion;
  u32 dwResetTime;                      // time to reset in seconds
  u32 dwRunTime;                        // Runtime in seconds
  u32 dwResetsCount;                    // count of resets
  u32 dwTriggerCount;                   // count of triggers
  u32 dwLastTriggerTime;                // time since last trigger
  u08 bBeep;                            // Beeper is on or off
  u08 bStopReset;                       // Reset or not reset
  u32 dwResetLogEntryCount;             // Count of log entries

} TGData;


class TGlobalData {
  TGData Data;

public:

  TGlobalData();
  ~TGlobalData();

  void ResetTime(u32 dwRT) { Data.dwResetTime = dwRT; }
  u32  ResetTime() { return Data.dwResetTime; }

  void RunTime(u32 dwRT) { Data.dwRunTime = dwRT; }
  u32  RunTime() { return Data.dwRunTime; }

  void ResetsCount(u32 dwRT) { Data.dwResetsCount = dwRT; }
  u32  ResetsCount() { return Data.dwResetsCount; }

  void TriggerCount(u32 dwRT) { Data.dwTriggerCount = dwRT; }
  u32  TriggerCount() { return Data.dwTriggerCount; }

  void LastTriggerTime(u32 dwRT) { Data.dwLastTriggerTime = dwRT; }
  u32  LastTriggerTime() { return Data.dwLastTriggerTime; }

  void WDGSWVersion( u16 Value) { Data.WDGSWVersion.value = Value; }
  u16  WDGSWVersion( ) { return Data.WDGSWVersion.value; }

  void Beep( u08 Value) { Data.bBeep = Value; }
  u08  Beep( ) { return Data.bBeep; }

  void StopReset( u08 Value) { Data.bStopReset = Value; }
  u08  StopReset( ) { return Data.bStopReset; }

  void ResetLogEntryCount( u32 Value) { Data.dwResetLogEntryCount = Value; }
  u32  ResetLogEntryCount( ) { return Data.dwResetLogEntryCount; }



};


/**************************************************************************
* declarations
**************************************************************************/


/**************************************************************************
* function prototypes
**************************************************************************/


/**************************************************************************
* implementation
**************************************************************************/




#undef GLOBAL
#endif // _GLOBALDATA_H
/*********************************** EOF *********************************/

