/***************************************************************
 * Name:      Logging.cpp
 * Purpose:   Logger functions
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/
#if !defined(_LOGGING_C)
#define _LOGGING_C



#include <string.h>
#include <stdio.h>

#include "types.h"
#include "Config.h"
#include "WDGProtocol.h"
#include "SoketComm.h"
#include "Logging.h"
#include "Watchdog.h"
#include "WdgCommProtocol.h"
#include "WDGComm.h"
#include "GlobalData.h"
#include "TimerClass.h"

#include "Externals.h"


/**************************************************************************
* definitions
**************************************************************************/


/**************************************************************************
* declarations
**************************************************************************/


/**************************************************************************
* function prototypes
**************************************************************************/
#define PRINTF(fmt, args...) printf(fmt, ##args)

/**************************************************************************
* implementation
**************************************************************************/
char * GetLevelString( TLogType bLevel )
{
  char * pszStr = (char *)"INFO ";
  switch( bLevel )
  {
    case LOG_INFO:
      pszStr = (char *)"INFO ";
    break;

    case LOG_ERROR:
      pszStr = (char *)"ERROR";
    break;

    case LOG_DEBUG:
      pszStr = (char *)"DEBUG";
    break;

    case LOG_NONE:
    default:
    break;
  }
  return pszStr;
}

void Log( TLogType bLevel, char * Str )
{
  if( (pGlobalConfig != NULL) && (bLevel > 0) )
  {
    if( bLevel <= pGlobalConfig->GetLogLevel() )
    {
      time_t rawtime;
      struct tm * timeinfo;
      FILE *LogFileStream;

      time ( &rawtime );
      timeinfo = localtime ( &rawtime );

      if( pGlobalConfig->LogToConsole() )
      {
          PRINTF( "%s: %02u/%02u/%4u %02u:%02u:%02u - %s\n",
                   GetLevelString(bLevel),
                   timeinfo->tm_mday,
                   timeinfo->tm_mon,
                   (timeinfo->tm_year + 1900),
                   timeinfo->tm_hour,
                   timeinfo->tm_min,
                   timeinfo->tm_sec,
                   Str
                  );
      }
      else
      {

        LogFileStream = fopen(pGlobalConfig->GetLogFileName(), "a");
        if ( LogFileStream != (FILE *)0 )
        {
          fprintf( LogFileStream, "%s: %02u/%02u/%4u %02u:%02u:%02u - %s\n",
                                   GetLevelString(bLevel),
                                   timeinfo->tm_mday,
                                   timeinfo->tm_mon,
                                   (timeinfo->tm_year + 1900),
                                   timeinfo->tm_hour,
                                   timeinfo->tm_min,
                                   timeinfo->tm_sec,
                                   Str
                                  );

          fclose( LogFileStream );
        }
      }
    }
  }
}


#endif /* */
/*********************************** EOF *********************************/
