/***************************************************************
 * Name:      SoketComm.cpp
 * Purpose:   Communication over sockets class
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/
#if !defined(__SOKETCOMM_CPP__)
#define __SOKETCOMM_CPP__

#include <stdio.h>
#include <unistd.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <sys/select.h>

#include <errno.h>


#include "Config.h"

#include "WDGProtocol.h"
#include "SoketComm.h"
#include "Logging.h"
#include "fcs.h"


/**************************************************************************
* definitions
**************************************************************************/


/**************************************************************************
* declarations
**************************************************************************/

TSocketComm * pSocketComm = NULL;


/**************************************************************************
* function prototypes
**************************************************************************/

#define RX_INDEX_IN(idx)  ( (WDGBuffIdx.bRxBufferIndex == 0 ? 1 : 0 ) * (sizeof(TWDGProtocol)) + idx )
#define TX_INDEX_OUT(idx) ( (WDGBuffIdx.bTxBufferIndex == 0 ? 1 : 0 ) * (sizeof(TWDGProtocol)) + idx )

#define RX_INDEX(idx) ( WDGBuffIdx.bRxBufferIndex * (sizeof(TWDGProtocol)) + idx )
#define TX_INDEX(idx) ( WDGBuffIdx.bTxBufferIndex * (sizeof(TWDGProtocol)) + idx )



/**************************************************************************
* implementation
**************************************************************************/

void * SocketThreadFunc( void * CallThread )
{
  TSocketComm * Thread = (TSocketComm *) CallThread;
  if( Thread != NULL )
  {
    Thread->ThreadFunc();
  }
  return NULL;
}

TSocketComm::TSocketComm( TConfig * pCfg )
{
  int iRet;
  pConfig  = NULL;

  bRXBuffer = new u08[ sizeof(TWDGProtocol) * 2 ];
  bTXBuffer = new u08[ sizeof(TWDGProtocol) * 2 ];

  memset( (u08 *)&WDGBuffIdx, 0, sizeof(TWDGBuffIdx) );

  if( (bRXBuffer != NULL) && (bTXBuffer != NULL) )
  {
    pConfig = pCfg;
  }

  if( pConfig != NULL )
  {
    dwLocalPort = pConfig->GetLocalPort();
    LOG(LOG_INFO, "LocalPort: %lu", dwLocalPort);

    void * pSelf  = this;
    fTermiate     = false;
    fRunning      = false;

    iRet = pthread_create( &SocketThreadID, NULL, SocketThreadFunc, pSelf);

    if( iRet == 0 )
    {
      LOG(LOG_INFO, "TSocketCom-> Socket Thread is created." );
      fReady = true;
    }
    else
    {
      LOG( LOG_ERROR, "TSocketCom-> Error on crating thread: %i : %s", iRet, (iRet == EAGAIN ? "EAGAIN" : (iRet == EINVAL ? "EINVAL" : ( iRet == EPERM ? "EPERM" : "Unknown error"))) );
    }
  }
}

TSocketComm::~TSocketComm()
{

  if( bRXBuffer != NULL )
  {
    delete bRXBuffer ;
  }

  if( bRXBuffer != NULL )
  {
    delete bTXBuffer ;
  }

}


u32 TSocketComm::Write( TWDGProtocol * pSrc )
{
  u32 dwRetVal = 0;
  TWDGProtocol LocalData;
  u16convert ewCRC;

  memcpy( (u08 *)&LocalData, pSrc, sizeof(TWDGProtocol) - 2 );
  ewCRC.value = CalculateCRC((u08 *)&LocalData, sizeof(TWDGProtocol) - 2 );
  LocalData.dwCRC.value = ewCRC.value;

  memcpy( &bTXBuffer[ TX_INDEX_OUT( 0 ) ], (u08 *)&LocalData, sizeof(TWDGProtocol) );
  if( WDGBuffIdx.fTxData == false )
  {
    WDGBuffIdx.bTxBufferIndex = WDGBuffIdx.bTxBufferIndex == 0 ? 1 : 0;
    dwRetVal = sizeof(TWDGProtocol);
    WDGBuffIdx.bTxIndex = 0;
    WDGBuffIdx.fTxData  = true;
  }

  return dwRetVal;
}

// reads data from internal buffer of WdgConnect class
u32 TSocketComm::Read( TWDGProtocol * pDst )
{
  u32 dwRetVal = 0;

  if( WDGBuffIdx.fRxData )
  {
    memcpy( pDst, &bRXBuffer[ RX_INDEX( 0 ) ],  sizeof(TWDGProtocol) );
    WDGBuffIdx.bRxBufferIndex = WDGBuffIdx.bRxBufferIndex == 1 ? 0 : 1;
    dwRetVal = sizeof(TWDGProtocol);

    WDGBuffIdx.bRxIndex = 0;
    WDGBuffIdx.fRxData = false;
  }

  return dwRetVal;
}

void TSocketComm::ThreadFunc( void )
{
  int sockfd;
  int ssock = -1;
  /* my address information, address where I run this program */
  struct sockaddr_in my_addr;
  /* remote address information */
  struct sockaddr_in their_addr;
  struct timeval tv;
  int nDelay;
  int alen;
  s32 dwBytesToRead  = 0;
  s32 dwBytesToWrite = 0;


  fd_set rfds, afds;

  LOG(LOG_DEBUG, "Starting socket thread");

  sockfd = socket(AF_INET, SOCK_STREAM, 0);

  if( sockfd != -1 )
  {

    /* host byte order */
    my_addr.sin_family = AF_INET;

    /* short, network byte order */
    my_addr.sin_port = htons( pConfig->GetLocalPort() );

    /* auto-fill with my IP */
    my_addr.sin_addr.s_addr = INADDR_ANY;
    /* zero the rest of the struct */
    memset( &(my_addr.sin_zero), 0, 8);

    if( bind( sockfd, (struct sockaddr *)&my_addr, sizeof(my_addr) ) != -1)
    {
      if( listen( sockfd, QLEN) != -1)
      {
        int yes = 1;
        if ( setsockopt( sockfd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(int)) != -1 )
        {
          fRunning = true;
        }
        else
        {
          // fTermiate = true;
          LOG(LOG_ERROR, "setsockopt() is failed");
        }

      }
      else
      {
        fTermiate = true;
        LOG(LOG_ERROR, "listen() is failed");
      }
    }
    else
    {
      fTermiate = true;
      LOG(LOG_ERROR, "bind() is failed");
    }
  }
  else
  {
    fTermiate = true;
    LOG(LOG_ERROR, "socket() is failed");
  }


  bool fAccepted = false;


  if( fRunning )
  {
    LOG(LOG_DEBUG, "Waiting for Data on port %lu", pConfig->GetLocalPort());
    FD_ZERO(&rfds);
    FD_SET( sockfd, &afds );
  }


  while( !fTermiate )
  {

    //if( !fAccepted )
    {
      memcpy(&rfds, &afds, sizeof(rfds));

      tv.tv_sec  = 0;
      tv.tv_usec = 200000;
      nDelay = select( sockfd + 1, &rfds, NULL, NULL, &tv );

      if (nDelay == -1)
      {
        LOG(LOG_ERROR, "Select on socket is failed");
        fTermiate = true;
      }
      else if ( nDelay )
      {
        if( ! fAccepted )
        {
          alen = sizeof(their_addr);
          ssock = accept( sockfd, (struct sockaddr *)&their_addr, (socklen_t *)&alen);
          if( ssock >= 0 )
          {
            fAccepted = true;

            LOG(LOG_INFO, "Inbound connection from %s", inet_ntoa(their_addr.sin_addr) );
          }
        }
      }
    }

    if ( fAccepted )
    {
      if( WDGBuffIdx.fTxData )                    // Is there some data to send
      {
        dwBytesToWrite = sizeof( TWDGProtocol ) - WDGBuffIdx.bTxIndex;
        if( dwBytesToWrite > 0 )
        {
          nTrnBytes = send( ssock, &bTXBuffer[ TX_INDEX( WDGBuffIdx.bTxIndex ) ], dwBytesToWrite, MSG_DONTWAIT );
          if( nTrnBytes >= 0 )
          {
            WDGBuffIdx.bTxIndex += nTrnBytes;
          }
          else
          {
            if( errno != EAGAIN )
            {
              LOG(LOG_ERROR, "send Error %s", strerror(errno));
              LOG(LOG_ERROR, "Connection closed");
              close( ssock );
              ssock = -1;
              fAccepted = false;
            }
          }
        }
        else
        {
          WDGBuffIdx.fTxData = false;
        }
      }

                                                                      // Receive some data
      if( ! WDGBuffIdx.fRxData )
      {

        dwBytesToRead = sizeof( TWDGProtocol ) - WDGBuffIdx.bRxIndex;                      // Calc count of bytes to read
        if( dwBytesToRead > 0 )                                                             // is there some data to read left
        {
          nRecBytes = recv( ssock, &bRXBuffer[ RX_INDEX( WDGBuffIdx.bRxIndex ) ], dwBytesToRead, MSG_DONTWAIT ); //
          if( nRecBytes > 0 )
          {
            WDGBuffIdx.bRxIndex += nRecBytes;
            if( WDGBuffIdx.bRxIndex >= sizeof( TWDGProtocol ) )                              // have we read everything
            {
              WDGBuffIdx.fRxData = true;                                                      // Set flag of it
            }
          }
          else if( nRecBytes == 0 )
          {
              LOG(LOG_INFO, "Connection closed");
              close( ssock );
              ssock = -1;
              fAccepted = false;
          }
          else
          {
            if( errno != EAGAIN )
            {
              LOG(LOG_ERROR, "recv Error %s", strerror(errno));
              LOG(LOG_ERROR, "Connection closed");
              close( ssock );
              ssock = -1;
              fAccepted = false;
            }
          }
        }
      }

    }
  }


  fRunning = false;

  if( ssock >= 0 )
  {
    close( ssock );
  }

  if( sockfd >= 0 )
  {
    close( sockfd );
  }

  LOG(LOG_DEBUG, "Socket thread ended");
}

void TSocketComm::Stop()
{
  if( fRunning == true )
  {
    fTermiate = true;
    pthread_join( SocketThreadID, NULL);
  }
}


#endif /* */
/*********************************** EOF *********************************/
