/***************************************************************
 * Name:      SoketComm.cpp
 * Purpose:   Communication over sockets class
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/
#if !defined(SOKETCOMM_H_INCLUDED)
#define SOKETCOMM_H_INCLUDED


#include <pthread.h>

#if defined(__SOKETCOMM_CPP__)
#define GLOBAL
#else
#define GLOBAL extern
#endif



/**************************************************************************
* definitions
**************************************************************************/

#define QLEN  128

#define max(x,y) ((x) > (y) ? (x) : (y))

#define  REC_BUF_LEN  128

typedef struct {
  u08 bTxBufferIndex;
  u08 bRxBufferIndex;

  u08 bTxIndex;
  u08 bRxIndex;

  u08 fRxData;
  u08 fTxData;

} TWDGBuffIdx;

class TSocketComm {

  TConfig * pConfig;
  u32 dwLocalPort;
  pthread_t SocketThreadID;

  bool fTermiate;

  bool fReady;
  bool fRunning;

  int nRecBytes;
  int nTrnBytes;
  u08 * bRXBuffer;
  u08 * bTXBuffer;
  TWDGBuffIdx WDGBuffIdx;

public:

  void ThreadFunc( void );

  TSocketComm( TConfig * pCfg );
  ~TSocketComm();
  void Stop();
  bool IsRunning() { return fRunning; }

  u32 Read( TWDGProtocol * pDst );
  u32 Write( TWDGProtocol * pSrc );
};

/**************************************************************************
* declarations
**************************************************************************/


/**************************************************************************
* function prototypes
**************************************************************************/





#undef GLOBAL
#endif /* SOKETCOMM_H_INCLUDED */
/*********************************** EOF *********************************/
