/***************************************************************
 * Name:      TimerClass.cpp
 * Purpose:   Software timer class
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/
#if !defined(_TIMERCLASS_C)
#define _TIMERCLASS_C


#include <stdio.h>
#include <string.h>

#include <sys/select.h>

#include "types.h"

#include "swtimer.h"
#include "TimerDefs.h"
#include "TimerClass.h"



/**************************************************************************
* definitions
**************************************************************************/


/**************************************************************************
* declarations
**************************************************************************/
TTimer * Timer = NULL;

/**************************************************************************
* function prototypes
**************************************************************************/


/**************************************************************************
* implementation
**************************************************************************/
void * TimerThreadFunc( void * CallThread )
{

  TTimer * Thread = (TTimer *) CallThread;
  if( Thread != NULL )
  {
    Thread->ThreadFunc();
  }
  return NULL;
}

TTimer::TTimer( )
{
  SWTInit( );

  void * pSelf  = this;

  fTermiate = false;
  fRunning  = false;

  int iRet = pthread_create( &TimerThreadID, NULL, TimerThreadFunc, pSelf);

  if( iRet == 0 )
  {
    fReady = true;
  }
}

TTimer::~TTimer()
{
  if( fRunning )
  {
    fTermiate = true;
    pthread_join( TimerThreadID, NULL);
  }
}

void TTimer::ThreadFunc(void)
{
  struct timeval tv;

  fRunning = true;

  while( !fTermiate )
  {
    tv.tv_sec  = 0;
    tv.tv_usec = 1000;
    select( 0, NULL, NULL, NULL, &tv );

    SWTTrigger();
  }

  fRunning  = false;
}


void TTimer::Load( u16 wTimerId, u32 dwTimerVal )
{
  SWTClear( wTimerId );
  SWTLoad( wTimerId, dwTimerVal );
}

bool TTimer::Flag( u16 wTimerId )
{
  return SWTFlag( wTimerId );
}

void TTimer::Clear( u16 wTimerId )
{
  SWTClear( wTimerId );
}

bool TTimer::Enabled( u16 wTimerId )
{
  return  SWTEnabled( wTimerId );
}

void TTimer::Stop( u16 wTimerId )
{
  SWTStop( wTimerId );
}
void TTimer::Resume( u16 wTimerId )
{
  SWTResume( wTimerId );
}






#endif /* _TIMERCLASS_C */
/*********************************** EOF *********************************/
