/***************************************************************
 * Name:      Watchdog.cpp
 * Purpose:   Watchdog class
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/
#if !defined(_WATCHDOG_C)
#define _WATCHDOG_C

#include <stdio.h>
#include <string.h>


#include "types.h"
#include "Config.h"

#include "WDGProtocol.h"
#include "Logging.h"

#include "Watchdog.h"
#include "SoketComm.h"
#include "WdgCommProtocol.h"
#include "WDGComm.h"
#include "GlobalData.h"
#include "TimerClass.h"

#include "Externals.h"


/**************************************************************************
* definitions
**************************************************************************/


/**************************************************************************
* declarations
**************************************************************************/


/**************************************************************************
* function prototypes
**************************************************************************/


/**************************************************************************
* implementation
**************************************************************************/

TWatchdog * Watchdog = NULL;

TWatchdog::TWatchdog()
{
  memset((u08 *)&WdgData, 0, sizeof(TWatchdogData) );
}

TWatchdog::~TWatchdog()
{

}

void TWatchdog::SetWDGLogEntry( TWD_LogEntry * SrcEntry )
{
  if( SrcEntry != NULL )
  {
    memcpy((u08 *)&WdgData.WDGLogEntry, (u08 * )SrcEntry, sizeof(TWD_LogEntry) );
  }
}

void TWatchdog::GetWDGLogEntry( TWD_LogEntry * DstEntry )
{
  if( DstEntry != NULL )
  {
    memcpy( (u08*)DstEntry, (u08 *)&WdgData.WDGLogEntry, sizeof(TWD_LogEntry) );
  }
}


void TWatchdog::Sock_MakeInfoPacket( TWDGProtocol * pDstPacket )
{
  if( (pDstPacket != NULL) && (GlobalData != NULL) && (pGlobalConfig != NULL) )
  {
    pDstPacket->Header.bCmd = WDG_CMD_GET_INFO;
    pDstPacket->Data.WDGInfo.dwRunTime          = GlobalData->RunTime();
    pDstPacket->Data.WDGInfo.dwResetsCount      = GlobalData->ResetsCount();
    pDstPacket->Data.WDGInfo.dwTriggerCount     = GlobalData->TriggerCount();
    pDstPacket->Data.WDGInfo.dwTriggerTime      = pGlobalConfig->GetTriggerDelay();
    pDstPacket->Data.WDGInfo.dwLastTriggerTime  = GlobalData->LastTriggerTime();
    pDstPacket->Data.WDGInfo.dwResetTime        = GlobalData->ResetTime();
    pDstPacket->Data.WDGInfo.bBeeping           = GlobalData->Beep( );
    pDstPacket->Data.WDGInfo.bStopReset         = GlobalData->StopReset();
    pDstPacket->Data.WDGInfo.dwLogEntryCount    = GlobalData->ResetLogEntryCount();

    memcpy((u08 *)pDstPacket->Data.WDGInfo.bCommPort, (u08 *)pGlobalConfig->GetComPortName(), CFG_COMPORT_NAME_LEN );
  }
}

void TWatchdog::Sock_MakeSettingsPacket( TWDGProtocol * pDstPacket )
{
  if( (pDstPacket != NULL) )
  {
    memset( (u08 *)pDstPacket, 0, sizeof(TWDGProtocol) );
    pDstPacket->Header.bCmd = WDG_CMD_SETTINGS;
  }
}

bool TWatchdog::Wdg_Trigger(void)
{
  bool fRes = false;
  if( WDConnect != NULL )
  {
    fRes = WDConnect->SendRequest( WD_CMD_TRIGGER, NULL );
  }
  return fRes;
}

void TWatchdog::Sock_MakeLogEntryPacket( TWDGProtocol * pDstPacket )
{
  TWD_LogEntry LogEntry;


  LOG(LOG_DEBUG, "Making LogEntry Packet");

  if( (pDstPacket != NULL) )
  {
    memset( (u08 *)pDstPacket, 0, sizeof(TWDGProtocol) );

    pDstPacket->Header.bCmd = WDG_CMD_GET_LOG_ENTRY;

    GetWDGLogEntry( &LogEntry );

    pDstPacket->Data.WDGLogEntry.dwIndex                    = LogEntry.dwIndex;
    pDstPacket->Data.WDGLogEntry.LogItem.dwID               = LogEntry.LogItem.dwID;
    pDstPacket->Data.WDGLogEntry.LogItem.dwThisResetCounter = LogEntry.LogItem.dwThisResetCounter;
    pDstPacket->Data.WDGLogEntry.LogItem.dwRunTimeAtReset   = LogEntry.LogItem.dwRunTimeAtReset;
    memcpy( (u08 *)&pDstPacket->Data.WDGLogEntry.LogItem.DateTimeAtReset, (u08 *)&LogEntry.LogItem.DateTimeAtReset, sizeof(TWDGDateTime) );

    LOG(LOG_DEBUG, "LogEntry %lu copied", pDstPacket->Data.WDGLogEntry.dwIndex );

  }
}

void TWatchdog::Sock_MakeLogClearedPacket( TWDGProtocol * pDstPacket )
{
  LOG(LOG_DEBUG, "Making Log Cleared Packet");

  if( (pDstPacket != NULL) )
  {
    memset( (u08 *)pDstPacket, 0, sizeof(TWDGProtocol) );

    pDstPacket->Header.bCmd = WDG_CMD_CLEAR_LOG;
  }
}


#endif /* _WATCHDOG_C */
/*********************************** EOF *********************************/
