/***************************************************************
 * Name:      Watchdog.h
 * Purpose:   Watchdog class
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/

#ifndef _WATCHDOG_H
#define _WATCHDOG_H


#include "WdgCommProtocol.h"


#if defined(_WATCHDOG_C)
#define GLOBAL
#else
#define GLOBAL extern
#endif

typedef struct {
  u08 aPortName[ WDGINFO_PORTNAMELEN + 1];
  u32 dwRunTime;
  u32 dwResetCount;
  u32 dwTriggerCount;
  u32 dwTriggerTime;
  u32 dwLastTriggerTime;

  TWD_LogEntry WDGLogEntry;

} PACKED TWatchdogData;


class TWatchdog {

  TWatchdogData WdgData;



public:
  TWatchdog();
  ~TWatchdog();

  void Sock_MakeInfoPacket( TWDGProtocol * pDstPacket );
  void Sock_MakeSettingsPacket( TWDGProtocol * pDstPacket );
  void Sock_MakeLogEntryPacket( TWDGProtocol * pDstPacket );
  void Sock_MakeLogClearedPacket( TWDGProtocol * pDstPacket );


  void SetWDGLogEntry( TWD_LogEntry * SrcEntry );
  void GetWDGLogEntry( TWD_LogEntry * DstEntry );


  bool Wdg_Trigger(void);
};

/**************************************************************************
* definitions
**************************************************************************/


/**************************************************************************
* declarations
**************************************************************************/


/**************************************************************************
* function prototypes
**************************************************************************/


/**************************************************************************
* implementation
**************************************************************************/




#undef GLOBAL
#endif // _WATCHDOG_H
/*********************************** EOF *********************************/

