/***************************************************************
 * Name:      main.cpp
 * Purpose:   Main function here
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/
#ifndef __MAIN_CPP_
#define __MAIN_CPP_

#include "struct.h"

#include <unistd.h>
#include <signal.h>
#include <stdio.h>


#include "types.h"
#include "Config.h"
#include "Init.h"
#include "main.h"
#include "TaskDefs.h"
#include "tasks.h"
#include "WDGProtocol.h"
#include "SoketComm.h"
#include "Logging.h"
#include "fcs.h"
#include "Watchdog.h"

#include "WdgCommProtocol.h"
#include "WDGComm.h"

#include "GlobalData.h"
#include "TimerDefs.h"
#include "TimerClass.h"

#include "Externals.h"


using namespace std;

bool gfDoIt = true;

void SigExit(int  signum)
{
  LOG(LOG_INFO, "Exiting");
  gfDoIt = false;
}

void DoAnswerTask(void);

void ProcessSWTimers(void);

int main()
{
  signal(SIGINT, SigExit);
  signal(SIGTERM, SigExit);


  if( Init() )
  {
    LOG(LOG_INFO, "PCWatchDog trigger Ver. %02u.%02u. Govorukhin Alexey. 2009", SOFT_VER_H, SOFT_VER_L);

    AddTask( TSK_MAIN_TASK, MT_OPEN_PORT );

    while( gfDoIt )
    {
      u08 bTaskId    = 0;
      bool bTaskDone = false;

      bTaskId = GetNextTask( TSK_MAIN_TASK );

      switch( bTaskId )
      {
        case MT_IDLE:

        break;

        case MT_OPEN_PORT:
          bTaskDone = true;

          if( WDConnect != NULL )
          {
            LOG(LOG_DEBUG, "Opening port");
            while( (WDConnect->Open() == -1) && ( gfDoIt ) )
            {

            }
          }

          if( WDConnect->IsOpened() )
          {
            AddTask( TSK_MAIN_TASK, MT_TRIGGER_WDG );
            Timer->Load( SWT_TRIGGER_TIMER, 10 );
          }
          else
          {
            LOG(LOG_ERROR, "Error on Opening port --> Exiting");
            gfDoIt = false;
          }

        break;

        case MT_TRIGGER_WDG:
          bTaskDone = true;
          LOG(LOG_DEBUG, "Triggering");
          if( Watchdog != NULL )
          {
            if( Watchdog->Wdg_Trigger() == false)
            {
              AddTask( TSK_MAIN_TASK, MT_OPEN_PORT );
            }
          }
        break;

        default:
          bTaskDone = true;
        break;
      }

      if( bTaskDone )
      {
        DoneTask( TSK_MAIN_TASK );
      }


      if( !pSocketComm->IsRunning() )
      {
        gfDoIt = false;
      }
      else
      {
        TWDGProtocol WDGData;

        // Socket communication
        if( pSocketComm->Read( &WDGData ) )
        {
          u16convert ewCRC;
          ewCRC.value = CalculateCRC((u08 *)&WDGData, sizeof(TWDGProtocol) - 2 );

          if( ewCRC.value == WDGData.dwCRC.value )
          {
            switch( WDGData.Header.bCmd )
            {
              case WDG_CMD_GET_INFO:
                LOG(LOG_DEBUG, "Get Info Cmd received");
                if( WDConnect != NULL )
                {
                  if( WDConnect->SendRequest( WD_CMD_INFO, NULL ) == false )
                  {
                    AddTask( TSK_MAIN_TASK, MT_OPEN_PORT );
                  }
                }
              break;

              case WDG_CMD_SETTINGS:
                LOG(LOG_DEBUG, "Settings Cmd received");
                if( WDGData.Data.WDGSettings.dwSettings & WDG_SET_MASK_TRIGGER_TIME )
                {
                  if( pGlobalConfig != NULL )
                  {
                    pGlobalConfig->SetTriggerDelay( WDGData.Data.WDGSettings.dwTriggerTime );
                  }
                }

                if( WDConnect != NULL )
                {
                  if( WDConnect->SendRequest( WD_CMD_SETINGS, (void *)&WDGData.Data.WDGSettings ) == false )
                  {
                    AddTask( TSK_MAIN_TASK, MT_OPEN_PORT );
                  }
                }

              break;

              case WDG_CMD_GET_LOG_ENTRY:
                LOG(LOG_DEBUG, "Get Log Entry %lu Cmd received", WDGData.Data.WDGLogEntry.dwIndex );

                if( WDConnect != NULL )
                {
                  if( WDConnect->SendRequest( WD_CMD_LOG, (void *)&WDGData.Data.WDGLogEntry ) == false )
                  {
                    AddTask( TSK_MAIN_TASK, MT_OPEN_PORT );
                  }
                }

              break;
              case WDG_CMD_CLEAR_LOG:
                LOG(LOG_DEBUG, "CLEAR Log Cmd received" );

                if( WDConnect != NULL )
                {
                  if( WDConnect->SendRequest( WD_CMD_CLEAR_LOG, NULL ) == false )
                  {
                    AddTask( TSK_MAIN_TASK, MT_OPEN_PORT );
                  }
                }
              break;

              default:
              break;
            }
          }
        }

      }

      DoAnswerTask();

      ProcessSWTimers();

    }

  }
  else
  {
    LOG(LOG_ERROR, "Error on Init");
  }

  Clear();
  return 0;
}

void ProcessSWTimers(void)
{
  if( (Timer != NULL) && (pGlobalConfig != NULL) )
  {
    if( Timer->Flag( SWT_TRIGGER_TIMER ) )
    {
      AddTask( TSK_MAIN_TASK, MT_TRIGGER_WDG );
      Timer->Load( SWT_TRIGGER_TIMER, pGlobalConfig->GetTriggerDelay() );
    }

    if( Timer->Flag( SWT_ANSWER_DELAY_TIMER ) )
    {
      Timer->Clear(  SWT_ANSWER_DELAY_TIMER );
      AddTask( TSK_MAIN_TASK, MT_OPEN_PORT );
    }
  }
}



void DoAnswerTask(void)
{
  u08 bTaskId    = 0;
  bool bTaskDone = false;
  TWDGProtocol pPacket;

  bTaskId = GetNextTask( TSK_ANSWER_TASK );

  switch( bTaskId )
  {
    case AT_NO_ANSWER:
      usleep(300);
    break;
    case AT_INFO:
      bTaskDone = true;
      if( Watchdog != NULL )
      {
        Watchdog->Sock_MakeInfoPacket( &pPacket );
        pSocketComm->Write( &pPacket );
      }
    break;

    case AT_SETTINGS:
      bTaskDone = true;
      if( Watchdog != NULL )
      {
        Watchdog->Sock_MakeSettingsPacket( &pPacket );
        pSocketComm->Write( &pPacket );
      }
    break;
    case AT_LOG_ENTRY:
      bTaskDone = true;
      if( Watchdog != NULL )
      {
        Watchdog->Sock_MakeLogEntryPacket( &pPacket );
        pSocketComm->Write( &pPacket );
      }
    break;
    case AT_LOG_CLEAR:
      bTaskDone = true;
      if( Watchdog != NULL )
      {
        Watchdog->Sock_MakeLogClearedPacket( &pPacket );
        pSocketComm->Write( &pPacket );
      }
    break;
    default:
      bTaskDone = true;

    break;
  }

  if( bTaskDone )
  {
    DoneTask( TSK_ANSWER_TASK );
  }
}

#endif

