/***************************************************************
 * Name:      Config.cpp
 * Purpose:   Configuration class
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/
#if !defined(__Config_c)
#define __Config_c

#include <wx/stdpaths.h>
#include <wx/fileconf.h>
#include "wx/timer.h"

#include "types.h"
#include "Config.h"

#include "WatchDogConnectMain.h"
#include "WatchdogClass.h"
#include "WDGProtocol.h"
#include "WdgConnect.h"

#include "Externals.h"

/**************************************************************************
* definitions
**************************************************************************/


/**************************************************************************
* declarations
**************************************************************************/


/**************************************************************************
* function prototypes
**************************************************************************/


/**************************************************************************
* implementation
**************************************************************************/

TConfig::TConfig(wxString wsFName)
{
  wxString Str;
  long lTempVal;
  u32 i;
  u32 dwCnt;
  wxStandardPaths Path;

  fReady = false;
  wsFileName = Path.GetExecutablePath() + wsFName; //  + wxT("/")

  Conf = new wxFileConfig("", "", wsFileName, "", wxCONFIG_USE_LOCAL_FILE);
  if( Conf != NULL )
  {
    fReady = true;

    Conf->SetPath(wxT("/"));

    Conf->Read(wxT("Options/ConnectDelay"), &Str, "100" );
    if( Str.ToLong( &lTempVal ) )
    {
      ConfigData.Options.dwConnectDelay = lTempVal;
    }

    Conf->Read(wxT("Options/PeriodicWDInfoRequest"), &Str, "0" );
    if( Str.ToLong( &lTempVal ) )
    {
      ConfigData.Options.bPeriodicWDInfoRequest = lTempVal;
    }

    Conf->Read(wxT("Options/PeriodicWDInfoRequestTimer"), &Str, "10" );
    if( Str.ToLong( &lTempVal ) )
    {
      ConfigData.Options.bPeriodicWDInfoRequestTimer = lTempVal;
    }

    Conf->Read(wxT("Watchdogs/Count"), &Str, "0" );
    if( Str.ToLong( &lTempVal ) )
    {
      if( Watchdogs != NULL )
      {
        wxString wsChapter;
        dwCnt = lTempVal;
        for( i = 0; i < dwCnt; i++ )
        {
          u32 dwIdx = 0;
          wsChapter.Printf("Watchdog%lu/Name", i);
          Conf->Read(wsChapter, &Str, "New WD" );
          dwIdx = Watchdogs->Add( (char *)Str.mb_str(wxConvUTF8) );

          wsChapter.Printf("Watchdog%lu/Address", i);
          Conf->Read(wsChapter, &Str, "0.0.0.0" );
          Watchdogs->SetAddress( dwIdx, (char *)Str.mb_str(wxConvUTF8) );

          wsChapter.Printf("Watchdog%lu/Port", i);
          Conf->Read(wsChapter, &Str, "0" );
          if( Str.ToLong( &lTempVal ) )
          {
            Watchdogs->SetPort( dwIdx, lTempVal );
          }
        }
      }
    }
  }
}


TConfig::~TConfig()
{
  if( Conf != NULL )
  {
    delete Conf;
    Conf = NULL;
  }
}

bool TConfig::Save(TDataTypes DT)
{
  u08 bRes = 1;

  if( Conf != NULL )
  {
    Conf->SetPath(wxT("/"));
    switch(DT)
    {
      case DT_COMM_SETTINGS:

      break;

      case DT_TERMINAL_SETTINGS:

      break;

      case DT_CONNECTION_SETTINGS:

      break;

      case DT_OPTIONS:

        bRes &= Conf->Write(wxT("Options/PeriodicWDInfoRequest"), (long)ConfigData.Options.bPeriodicWDInfoRequest );
        bRes &= Conf->Write(wxT("Options/PeriodicWDInfoRequestTimer"), (long)ConfigData.Options.bPeriodicWDInfoRequestTimer );
        bRes &= Conf->Write(wxT("Options/ConnectDelay"), (long)ConfigData.Options.dwConnectDelay );

      break;
      case DT_WATCHDOGSETTINGS:
        if( Watchdogs != NULL )
        {
          wxString wsChapter;
          wxString wsParameter;
          u32 dwWDCount = Watchdogs->GetCount();

          bRes &= Conf->Write(wxT("Watchdogs/Count"), (long)dwWDCount );

          u32 i;
          for( i = 0; i < dwWDCount; i++ )
          {
            wsChapter.Printf("Watchdog%lu/Name", i);
            wsParameter = Watchdogs->GetName(i);
            bRes &= Conf->Write(wsChapter, wsParameter );

            wsChapter.Printf("Watchdog%lu/Address", i);
            wsParameter = Watchdogs->GetAddress(i);
            bRes &= Conf->Write(wsChapter, wsParameter );

            wsChapter.Printf("Watchdog%lu/Port", i);
            wsParameter.Printf("%lu", Watchdogs->GetPort(i));
            bRes &= Conf->Write(wsChapter, wsParameter );
          }
        }
      break;

      default:
      break;
    }
    Conf->Flush();
  }
  else
  {
    bRes = 0;
  }

  return bRes;
}




#endif /* __Config_c */
/*********************************** EOF *********************************/
