/***************************************************************
 * Name:      GlobalData.cpp
 * Purpose:   Global Data class (actually Watchdog Data Class )
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/
#if !defined(_GLOBALDATA_C)
#define _GLOBALDATA_C

#include <stdio.h>
#include <string.h>
#include "types.h"
#include "GlobalData.h"


/**************************************************************************
* definitions
**************************************************************************/


/**************************************************************************
* declarations
**************************************************************************/

TWDGData * GlobalData = NULL;

/**************************************************************************
* function prototypes
**************************************************************************/


/**************************************************************************
* implementation
**************************************************************************/





#include <wx/listimpl.cpp>

WX_DEFINE_LIST( TWDGLog );


TWDGLogItemA::TWDGLogItemA( u32 dwNewId )
{
  dwId = dwNewId;
  memset( &LogItem, 0, sizeof(TWDLogItemClient) );
}


TWDGData::TWDGData()
{
  memset( (u08 *)&Data, 0, sizeof(TGlobData) );

  pWDGLog = new TWDGLog;

}

TWDGData::~TWDGData()
{

  if( pWDGLog != NULL )
  {
    TWDGLog::iterator iter;

    for (iter = pWDGLog->begin(); iter != pWDGLog->end(); ++iter)
    {
      TWDGLogItem *current = *iter;

      if( current != NULL )
      {
        delete current;
      }
      pWDGLog->DeleteObject( current );
    }

    delete pWDGLog;
    pWDGLog = NULL;
  }

}

void TWDGData::CommPort( u08 * pszStr )
{
  memset( Data.bCommPort, 0, WDGINFO_PORTNAMELEN + 1 );
  strncpy( (char *)Data.bCommPort, (char *)pszStr, WDGINFO_PORTNAMELEN );
}

u08 * TWDGData::CommPort( void )
{
  return Data.bCommPort;
}

void TWDGData::SetValue( TDataValue DV, u32 dwValue )
{
  u08 * pStr;
  switch( DV )
  {
    case GDV_PORT_NAME:
      pStr = (u08 *)dwValue;
      memset( Data.bCommPort, 0, WDGINFO_PORTNAMELEN + 1 );
      memcpy( Data.bCommPort, pStr, WDGINFO_PORTNAMELEN );
    break;

    case GDV_RUN_TIME:
      Data.dwRunTime = dwValue;
    break;

    case GDV_RESET_COUNT:
      Data.dwResetsCount = dwValue;
    break;

    case GDV_TRIGGER_COUNT:
      Data.dwTriggerCount = dwValue;
    break;

    case GDV_TRRIGGER_TIME:
      Data.dwTriggerTime = dwValue;
    break;

    case GDV_LAST_TRIGGER_TIME:
      Data.dwLastTriggerTime = dwValue;
    break;

    case GDV_RESET_TIME:
      Data.dwResetTime = dwValue;
    break;

    case GDV_BEEPER:
      Data.bBeeper = dwValue;
    break;

    case GDV_STOP_RESET:
      Data.bStopReset = dwValue;
    break;
    case GDV_LOG_ENTRY_COUNT:
      Data.dwResetLogEntryCount = dwValue;
    break;

    default:
    break;
  }
}


u32 TWDGData::GetValue( TDataValue DV )
{
  u32 dwRetVal = 0;
  switch( DV )
  {
    case GDV_PORT_NAME:
      dwRetVal = (u32) Data.bCommPort;
    break;

    case GDV_RUN_TIME:
      dwRetVal = Data.dwRunTime;
    break;

    case GDV_RESET_COUNT:
      dwRetVal = Data.dwResetsCount;
    break;

    case GDV_TRIGGER_COUNT:
      dwRetVal = Data.dwTriggerCount;
    break;

    case GDV_TRRIGGER_TIME:
      dwRetVal = Data.dwTriggerTime;
    break;

    case GDV_LAST_TRIGGER_TIME:
      dwRetVal = Data.dwLastTriggerTime;
    break;

    case GDV_RESET_TIME:
      dwRetVal = Data.dwResetTime;
    break;

    case GDV_BEEPER:
      dwRetVal = Data.bBeeper;
    break;

    case GDV_STOP_RESET:
      dwRetVal = Data.bStopReset;
    break;

    case GDV_LOG_ENTRY_COUNT:
      dwRetVal = Data.dwResetLogEntryCount;
    break;

    default:
      dwRetVal = 0;
    break;
  }
  return dwRetVal;
}

TWDLogItemClient * TWDGData::GetLogItem( u32 dwItemIdx )
{
  TWDLogItemClient * TempLI = NULL;
  if( (pWDGLog != NULL) )
  {
    TWDGLog::iterator iter;
    u32 dwIndex = 0;

    if( dwItemIdx < pWDGLog->GetCount() )
    {
      for (iter = pWDGLog->begin(); iter != pWDGLog->end(); ++iter)
      {
        TWDGLogItem *current = *iter;

        if( dwIndex == dwItemIdx )
        {
          if( current != NULL )
          {
            TempLI = &current->LogItem;
          }
          break;
        }
        dwIndex++;
      }
    }
  }

  return TempLI;
}

u32 TWDGData::GetLogItemsCount()
{
  u32 dwRetVal = 0;

  if(pWDGLog != NULL)
  {
    dwRetVal = pWDGLog->GetCount();
  }

  return dwRetVal;
}

u32 TWDGData::AddLogItem( TWDLogItemClient * pLogItem )
{
  u32 dwRetVal = (u32)-1;

  if( (pWDGLog != NULL) && (pLogItem != NULL) )
  {
    u32 count = pWDGLog->GetCount();

    TWDGLogItem * pLI = new TWDGLogItem( count );
    if( pLI != NULL )
    {
      pLI->dwId = count;
      memcpy( &pLI->LogItem, pLogItem, sizeof(TWDLogItemClient) );
      pWDGLog->Append( pLI );
      dwRetVal = count;
    }

  }
  return dwRetVal;
}

void TWDGData::DeleteLogItem( u32 dwItemIdx )
{
  if( (pWDGLog != NULL) )
  {
    TWDGLog::iterator iter;
    u32 dwIndex = 0;

    if( dwItemIdx < pWDGLog->GetCount() )
    {
      for (iter = pWDGLog->begin(); iter != pWDGLog->end(); ++iter)
      {
        TWDGLogItem *current = *iter;

        if( dwIndex == dwItemIdx )
        {
          if( current != NULL )
          {
            delete current;
          }
          pWDGLog->DeleteObject( current );
          break;
        }
        dwIndex++;
      }
    }
  }
}

void TWDGData::ClearLog( )
{
  if( pWDGLog != NULL )
  {
    TWDGLog::iterator iter;

    for (iter = pWDGLog->begin(); iter != pWDGLog->end(); ++iter)
    {
      TWDGLogItem *current = *iter;

      if( current != NULL )
      {
        delete current;
      }
      pWDGLog->DeleteObject( current );
    }
  }
}


#endif /* __%-r%_%-e%ml2%dc1%mr2 */
/*********************************** EOF *********************************/
