/***************************************************************
 * Name:      GlobalData.h
 * Purpose:   Global Data class (actually Watchdog Data Class )
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/
#if !defined(_GLOBALDATA_H)
#define _GLOBALDATA_H

#include <wx/list.h>


#include "WDGprotocol.h"

#ifdef _GLOBALDATA_C
#define GLOBAL
#else
#define GLOBAL extern
#endif


/**************************************************************************
* definitions
**************************************************************************/

typedef struct TWDGLogItemA {
  u32 dwId;
  TWDLogItemClient LogItem;

  TWDGLogItemA( u32 dwNewId );

} TWDGLogItem;


typedef enum {
 GDV_PORT_NAME,
 GDV_RUN_TIME,
 GDV_RESET_COUNT,
 GDV_TRIGGER_COUNT,
 GDV_TRRIGGER_TIME,
 GDV_LAST_TRIGGER_TIME,
 GDV_RESET_TIME,
 GDV_BEEPER,
 GDV_STOP_RESET,
 GDV_LOG_ENTRY_COUNT,

 GDV_MAX_VALUES
} TDataValue;

typedef struct {
  u08 bCommPort[ WDGINFO_PORTNAMELEN + 1 ]; // Name of com port
  u32 dwRunTime;                        // Runtime in seconds
  u32 dwResetsCount;                    // count of resets
  u32 dwTriggerCount;                   // count of triggers
  u32 dwTriggerTime;                    // Trigger delay time
  u32 dwLastTriggerTime;                // time since last trigger
  u32 dwResetTime;                      // Time without reset
  u08 bBeeper;                          // Beeper is used
  u08 bStopReset;                       // Reset or not reset
  u32 dwResetLogEntryCount;

} TGlobData;


WX_DECLARE_LIST(TWDGLogItem, TWDGLog);

class TWDGData {
  TGlobData Data;

  TWDGLog * pWDGLog;

public:
  TWDGData();
  ~TWDGData();

  void CommPort( u08 * pszStr );
  u08 * CommPort( void );

  void RunTime( u32 dwTime) { Data.dwRunTime = dwTime; }
  u32 RunTime( void ) { return Data.dwRunTime; }

  void ResetsCount( u32 dwVar ) { Data.dwResetsCount = dwVar; }
  u32 ResetsCount( void ) { return Data.dwResetsCount; }

  void TriggerCount( u32 dwVar ) { Data.dwTriggerCount = dwVar; }
  u32 TriggerCount( void ) { return Data.dwTriggerCount; }

  void TriggerTime( u32 dwVar ) { Data.dwTriggerTime = dwVar; }
  u32 TriggerTime( void ) { return Data.dwTriggerTime; }

  void LastTriggerTime( u32 dwVar ) { Data.dwLastTriggerTime = dwVar; }
  u32 LastTriggerTime( void ) { return Data.dwLastTriggerTime; }

  void ResetTime( u32 dwVar) { Data.dwResetTime = dwVar; }
  u32 ResetTime( void ) { return Data.dwResetTime; }

  void Beeper( u08 bVar) { Data.bBeeper = bVar; }
  u08 Beeper( void ) { return Data.bBeeper; }

  void StopReset( u08 bVar) { Data.bStopReset = bVar; }
  u08 StopReset( void ) { return Data.bStopReset; }

  void ResetLogEntryCount( u32 dwVar) { Data.dwResetLogEntryCount = dwVar; }
  u32 ResetLogEntryCount( void ) { return Data.dwResetLogEntryCount; }


  void SetValue( TDataValue DV, u32 dwValue );
  u32  GetValue( TDataValue DV );

  TWDLogItemClient * GetLogItem( u32 dwItemIdx );
  u32  GetLogItemsCount();
  u32  AddLogItem( TWDLogItemClient * pLogItem );
  void DeleteLogItem( u32 dwItemIdx );
  void ClearLog( );

};



/**************************************************************************
* declarations
**************************************************************************/


/**************************************************************************
* function prototypes
**************************************************************************/





#undef GLOBAL
#endif // _GLOBALDATA_H
/*********************************** EOF *********************************/


