/***************************************************************
 * Name:      WDGProtocol.h
 * Purpose:   Watchdog communications protocol definitions
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/
#ifndef _WDGPROTOCOL_H
#define _WDGPROTOCOL_H

#define WDGINFO_PORTNAMELEN 20

#define PACKED __attribute__((packed))

typedef struct {
  u08 bSec;
  u08 bMin;
  u08 bHour;

  u08 bDay;
  u08 bMonth;
  u08 bYear;      // + 2000

} TWDGDateTimeClient;

typedef enum {
  WDG_CMD_GET_INFO,
  WDG_CMD_SETTINGS,
  WDG_CMD_GET_LOG_ENTRY,
  WDG_CMD_CLEAR_LOG,

  WDG_CMD_MAX_CMDS
} TWDG_Cmds;

typedef enum {
  WDGS_GET_SETTINGS,
  WDGS_SET_SETTINGS,
  WDGS_MAX_SETTINGS_CMDS
} TWDG_SettingsCmds;



// Header
typedef struct {
  u08 bCmd;
} PACKED TWDGProt_Header;


// Information about watchdog
typedef struct {
  u08 bCommPort[ WDGINFO_PORTNAMELEN ]; // Name of com port
  u32 dwRunTime;                        // Runtime in seconds
  u32 dwResetsCount;                    // count of resets
  u32 dwTriggerCount;                   // count of triggers
  u32 dwTriggerTime;                    // Trigger delay time
  u32 dwLastTriggerTime;                // time since last trigger
  u32 dwResetTime;                      // Time to wait till reset
  u08 bBeeping;                         // Beep is on or off
  u08 bStopReset;                       // Reset or not reset
  u32 dwLogEntryCount;                  // count of entries in log

} PACKED TWDGInfo;


#define WDG_SET_MASK_RESET_TIME         0x00000001
#define WDG_SET_MASK_BEEP               0x00000002
#define WDG_SET_MASK_STOP_RESET         0x00000004
#define WDG_SET_MASK_RESET_IT           0x00000008
#define WDG_SET_MASK_CLEAR_CNT          0x00000010
#define WDG_SET_MASK_TRIGGER_TIME       0x00000020

// Watchdog settings
typedef struct {
  u08 bSettingsCmd;                     // Command to settings (set or get)

  u32 dwSettings;                       // Bit field what to set

  u32 dwResetTime;                      // Time to wait till reset
  u08 bBeep;                            // set Beeper on or off
  u08 bStopReset;                       // Stop or Start to reset
  u08 bResetIt;                         // Do Reset
  u08 bClearCounters;                   // Clear all counters
  u32 dwTriggerTime;                    // Trigger Time

} PACKED TWDGSettings;


typedef struct {
  u32 dwID;                             // 4
  u32 dwThisResetCounter;               // 4
  u32 dwRunTimeAtReset;                 // 4
  TWDGDateTimeClient DateTimeAtReset;   // 6
} PACKED TWDLogItemClient;              //   18


typedef struct {
  u32       dwIndex;
  TWDLogItemClient  LogItem;

} PACKED TWD_LogEntryClient;


// Packet
typedef struct {

  TWDGProt_Header Header;

  union {
    TWDGInfo            WDGInfo;
    TWDGSettings        WDGSettings;
    TWD_LogEntryClient  WDGLogEntry;
  } PACKED Data;

  u16convert dwCRC;

} PACKED TWDGProtocol;





#endif // _WDGPROTOCOL_H
