/***************************************************************
 * Name:      WDSettingsDialog.cpp
 * Purpose:   Watchdog settings Dialog class
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/

#include <wx/fileconf.h>
#include "wx/timer.h"
#include "wx/socket.h"

#include "types.h"

#include "WatchDogConnectMain.h"
#include "WDSettingsDialog.h"
#include "WatchdogClass.h"
#include "Config.h"
#include "WDGProtocol.h"
#include "WdgConnect.h"

#include "Externals.h"

WDSSetDlg::WDSSetDlg( wxWindow* parent ) : WDSettingsDialog(parent)
{
  int width;
  int height;

  memset( &Changes, 0, sizeof(TChanges) );

  WDsListBox->GetClientSize( &width, &height );

  WDsListBox->SetColumnWidth( 0, width );

  if( Watchdogs != NULL )
  {
    u32 dwCnt = Watchdogs->GetCount();
    u32 i;
    wxListItem MyLI;
    wxString wsTmp;
    char * pszStr;
    for( i = 0; i < dwCnt; i++ )
    {
      pszStr = Watchdogs->GetName( i );
      wsTmp = pszStr;
      MyLI.SetText( wsTmp );
      MyLI.SetId( i );
      WDsListBox->InsertItem( MyLI );
    }
  }
}

WDSSetDlg::~WDSSetDlg( )
{

}

void WDSSetDlg::WDSettingsCloseButtonOnClick( wxCommandEvent& event )
{
  Close();
  event.Skip();
}

void WDSSetDlg::NewWDButtonOnClick( wxCommandEvent& event )
{
  if( Watchdogs != NULL )
  {
    u32 dwIdx = Watchdogs->Add("New Watchdog");
    wxString wsTmp = _T("New Watchdog");
    if( dwIdx < MAX_U32 )
    {
      wxListItem MyLI;
      MyLI.SetText( wsTmp );
      MyLI.SetId( dwIdx );

      WDsListBox->InsertItem( MyLI );
    }
  }

  event.Skip();
}

void WDSSetDlg::CopyWDButtonOnClick( wxCommandEvent& event )
{

  event.Skip();
}

void WDSSetDlg::WDSettingsSaveButtonOnClick( wxCommandEvent& event )
{
  if( Config != NULL )
  {
    Config->Save( DT_WATCHDOGSETTINGS );
  }
  event.Skip();
}


void WDSSetDlg::WDsListBoxOnSelected( wxListEvent& event )
{
  long nItem = -1;
  u32 dwCount;

  dwCount = Watchdogs->GetCount();

  nItem = WDsListBox->GetNextItem(nItem, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);

  if( nItem != -1 )
  {
    Changes.dwCurIndex = nItem;

    wxString wsTempStr;
    wsTempStr.Printf(wxT("%lu/%lu"), nItem, dwCount );
    SelectedWDIDTC->SetValue( wsTempStr );

    char * pszStr = Watchdogs->GetName( nItem );
    wsTempStr = pszStr;
    WDNameTC->SetValue( wsTempStr );

    pszStr = Watchdogs->GetAddress( nItem );
    wsTempStr = pszStr;
    WDServerAddressTC->SetValue( wsTempStr );

    wsTempStr.Printf("%lu", Watchdogs->GetPort( nItem ) );
    WDServerPortTC->SetValue( wsTempStr );
  }

  event.Skip();
}

void WDSSetDlg::WDNameTCOnText( wxCommandEvent& event )
{
  Changes.bCHName = 1;
  event.Skip();
}
void WDSSetDlg::WDServerAddressTCOnText( wxCommandEvent& event )
{
  Changes.bCHAddress = 1;
  event.Skip();
}
void WDSSetDlg::WDServerPortTCOnText( wxCommandEvent& event )
{
  Changes.bCHPort = 1;
  event.Skip();
}

void WDSSetDlg::UpdateWDButtonOnClick( wxCommandEvent& event )
{
  // long nItem = -1;
  wxListItem MyLI;

  if( Changes.bCHName == 1 )
  {
    wxString wsName = WDNameTC->GetValue();
    Watchdogs->SetName( Changes.dwCurIndex, (char *)wsName.mb_str(wxConvUTF8) );

    MyLI.SetId( Changes.dwCurIndex );
    if( WDsListBox->GetItem( MyLI ) )
    {
      wsName = Watchdogs->GetName( Changes.dwCurIndex );
      MyLI.SetText( wsName );
      WDsListBox->SetItem( MyLI );
      Changes.bCHName = 0;
    }
  }

  if( Changes.bCHAddress == 1 )
  {
    wxString wsName = WDServerAddressTC->GetValue();
    Watchdogs->SetAddress( Changes.dwCurIndex, (char *)wsName.mb_str(wxConvUTF8) );
    Changes.bCHAddress = 0;
  }

  if( Changes.bCHPort == 1 )
  {
    wxString wsName = WDServerPortTC->GetValue();
    u32 dwPort = 0;
    if( wsName.ToLong( (long int *)&dwPort, 10 ) )
    {
      Watchdogs->SetPort( Changes.dwCurIndex, dwPort );
      Changes.bCHPort = 0;
    }
  }

  event.Skip();
}

void WDSSetDlg::DeleteWDButtonOnClick( wxCommandEvent& event )
{
  long nItem = -1;

  nItem = WDsListBox->GetNextItem(nItem, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
  if( nItem != -1 )
  {
    Watchdogs->Delete( nItem );
    WDsListBox->DeleteItem( nItem );
  }
  event.Skip();
}
