/***************************************************************
 * Name:      WatchDogConnectMain.cpp
 * Purpose:   WatchDogConnect Main frame class
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#ifdef __BORLANDC__
#pragma hdrstop
#endif //__BORLANDC__

#include <wx/fileconf.h>
#include "wx/msgdlg.h"
#include "wx/dialog.h"
#include "wx/timer.h"
#include "wx/socket.h"

#include "types.h"
#include "WatchDogConnectMain.h"
#include "WDSettingsDialog.h"
#include "WatchdogClass.h"
#include "WDGProtocol.h"
#include "WdgConnect.h"

#include "Config.h"

#include "GlobalData.h"

#include "Externals.h"

#include "DateTime.h"

#include "Options.h"

TWatchdog * Watchdogs = NULL;

TConfig * Config = NULL;

TWdgConnect * WdgConnect = NULL;


//helper functions
enum wxbuildinfoformat {
    short_f, long_f };

wxString wxbuildinfo(wxbuildinfoformat format)
{
    wxString wxbuild(wxVERSION_STRING);

    if (format == long_f )
    {
#if defined(__WXMSW__)
        wxbuild << _T("-Windows");
#elif defined(__WXMAC__)
        wxbuild << _T("-Mac");
#elif defined(__UNIX__)
        wxbuild << _T("-Linux");
#endif

#if wxUSE_UNICODE
        wxbuild << _T("-Unicode build");
#else
        wxbuild << _T("-ANSI build");
#endif // wxUSE_UNICODE
    }

    return wxbuild;
}


WatchDogConnectFrame::WatchDogConnectFrame(wxFrame *frame)
    : TMainFrame(frame)
{
#if wxUSE_STATUSBAR
    statusBar->SetStatusText(_("Hello Code::Blocks user!"), 0);
    statusBar->SetStatusText(wxbuildinfo(short_f), 1);
#endif
  GlobalData = new TWDGData;

  Watchdogs = new TWatchdog;

  Config = new TConfig( wxT("Settings.ini") );

  MainTimer = new wxTimer(this, MainTIMER_ID);
  MainTimer->Start(50);

	this->Connect( MainTIMER_ID, wxEVT_TIMER, wxTimerEventHandler( WatchDogConnectFrame::OnMainTimer ) );


  pWDSocket = new wxSocketClient();

  if( pWDSocket  != NULL )
  {
    pWDSocket->SetNotify( wxSOCKET_CONNECTION_FLAG | wxSOCKET_INPUT_FLAG | wxSOCKET_LOST_FLAG );
    pWDSocket->Notify( true );
    pWDSocket->SetEventHandler( (wxFrame &)*this, WD_SOCKET_ID );
    this->Connect( WD_SOCKET_ID, wxEVT_SOCKET, wxSocketEventHandler( WatchDogConnectFrame::OnSocketEvent ) );
  }


}


WatchDogConnectFrame::~WatchDogConnectFrame()
{

  if( WdgConnect != NULL )
  {
    WdgConnect->Delete();
  }

  if( pWDSocket != NULL )
  {
    this->Disconnect( wxEVT_SOCKET, wxSocketEventHandler( WatchDogConnectFrame::OnSocketEvent ) );

    pWDSocket->Destroy();

    delete pWDSocket;
    pWDSocket = NULL;
  }



  this->Disconnect( wxEVT_TIMER, wxTimerEventHandler( WatchDogConnectFrame::OnMainTimer ) );

  if(MainTimer != NULL)
  {
    delete MainTimer;
    MainTimer = NULL;
  }

  if( Watchdogs != NULL )
  {
    delete Watchdogs;
    Watchdogs = NULL;
  }

  if( Config != NULL )
  {
    delete Config;
    Config = NULL;
  }

  if( GlobalData != NULL )
  {
    delete GlobalData;
    GlobalData = NULL;
  }
}

void WatchDogConnectFrame::OnClose(wxCloseEvent &event)
{
    Destroy();
}

void WatchDogConnectFrame::OnQuit(wxCommandEvent &event)
{
    Destroy();
}

void WatchDogConnectFrame::OnAbout(wxCommandEvent &event)
{
    wxString msg = wxbuildinfo(long_f);
    wxMessageBox(msg, _("Welcome to..."));
}

void WatchDogConnectFrame::WatchdogsSettingsMIOnClick( wxCommandEvent& event )
{
  WDSSetDlg * WDSetDialog = new WDSSetDlg( (wxWindow *)this ); // , wxID_WD_SETTINGS_DIALOG, _T("Watchdogs settings")

  WDSetDialog->ShowModal();

  // event.Skip();
}

void WatchDogConnectFrame::miOptionsMIOnClick( wxCommandEvent& event )
{
  TOptionsDlg * OptDlg = new TOptionsDlg( this );

  OptDlg->Show();
}


void WatchDogConnectFrame::OnMainTimer(wxTimerEvent& event)
{
  u32 i;
  static u32 dwTimerCntr = 0;

  dwTimerCntr++;

  if( Watchdogs != NULL )
  {
    if( Watchdogs->GetChanged() )
    {
      WatchDogCB->Clear();
      for( i = 0; i < Watchdogs->GetCount(); i++ )
      {
        wxString wsTmpStr = Watchdogs->GetName( i );
        WatchDogCB->Append( wsTmpStr );
      }
      WatchDogCB->Select( 0 );
    }

    if( ( Watchdogs->GetChanged() == true ) || ( Watchdogs->NewData() == true ) )
    {
      Watchdogs->ResetChanged();
      Watchdogs->NewData( false );
      ShowWDInfo();
    }
    else
    {
      if( (dwTimerCntr % 20) == 0 )
      {
        ShowWDInfo( false );
      }
    }



  }

  /*
  if( WdgConnect != NULL )
  {
    WdgConnect->Communication();
  }
  */


  //
  if( Config != NULL )
  {
    if( (dwTimerCntr % ( (20) * Config->ConfigData.Options.bPeriodicWDInfoRequestTimer ) ) == 0 )
    {
      if( Config->ConfigData.Options.bPeriodicWDInfoRequest == 1 )
      {
        SendCmdGetInfo();
      }
    }
  }

}

void WatchDogConnectFrame::WatchDogCBOnCombobox( wxCommandEvent& event )
{
  ShowWDInfo();
  event.Skip();
}



void WatchDogConnectFrame::ShowWDInfo( bool ShowAll )
{
  u32 dwIdx;
  wxString wsCaption;

  if( Watchdogs != NULL )
  {
    dwIdx = WatchDogCB->GetCurrentSelection();

    wsCaption.Printf("Address: %s \n"
                     "Port:    %lu", Watchdogs->GetAddress( dwIdx ), Watchdogs->GetPort( dwIdx ) );
    WatchdogInfoST->SetLabel( wsCaption );
    TWDGData * pData = Watchdogs->GetWDGData( dwIdx );
    if( pData != NULL )
    {
      if( ShowAll )
      {
        char * pPortName = (char *)pData->CommPort();
        wsCaption.Printf( "%s", pPortName );
        WDGCommPortTC->SetValue( wsCaption );

        WDGTriggerTimeSpin->SetValue( pData->TriggerTime() );
      }

      TDateTimeW DTW;
      SecondsToDateTime( &DTW, pData->RunTime() );

      wsCaption.Printf(  "%lu s: %u y %u m %u d, %u h %u m %u s",
                          pData->RunTime(),
                          (u08)DTW.bYear,
                          (u08)DTW.bMonth,
                          (u08)DTW.bDay,
                          (u08)DTW.bHour,
                          (u08)DTW.bMin,
                          (u08)DTW.bSec
                      );
      WDGRunTimeLabel->SetLabel( wsCaption );


      wsCaption.Printf( "%lu", pData->TriggerCount() );
      WDGTriggersCountLabel->SetLabel( wsCaption );

      wsCaption.Printf( "%lu ms", pData->LastTriggerTime() );
      WDGLastTriggerTimeLabel->SetLabel( wsCaption );

      wsCaption.Printf( "%lu", pData->ResetsCount() );
      WDGResetsCountLabel->SetLabel( wsCaption );

      if( ShowAll )
      {
        ResetTimeSpin->SetValue( pData->ResetTime() );

        u08 bBeep = pData->Beeper();
        BeeperCB->SetValue( bBeep == 1 );

        u08 bStopReset = pData->StopReset();
        StopResetsCB->SetValue( bStopReset == 1 );
      }

      if( !Watchdogs->GettingLogProcess() )
      {
        wsCaption.Printf( "%lu entr%s", pData->ResetLogEntryCount(), pData->ResetLogEntryCount() == 1 ? "y" : "ies" );
        SetLogItemsCountText( wsCaption );
        WDGLogStartEntrySpin->SetRange(1, pData->ResetLogEntryCount() );

        GetLogButton->SetLabel(_("Get Log"));

        if( ShowAll )
        {
          TWDLogItemClient * pLogItem;
          RecentResetsLB->Clear();

          for( u32 i = 0 ; i < pData->GetLogItemsCount(); i++ )
          {
            pLogItem = pData->GetLogItem( i );
            if( pLogItem != NULL )
            {
              wxString wsStr;
              wsStr.Printf(_("%04lu: Cntr %04lu, Run Time %9lu s at %02u/%02u/%4u %02u:%02u:%02u"),

                              pLogItem->dwID,
                                         pLogItem->dwThisResetCounter,
                                                        pLogItem->dwRunTimeAtReset,
                                                               pLogItem->DateTimeAtReset.bDay + 1,
                                                                  pLogItem->DateTimeAtReset.bMonth + 1,
                                                                     pLogItem->DateTimeAtReset.bYear + 2000,
                                                                        pLogItem->DateTimeAtReset.bHour,
                                                                           pLogItem->DateTimeAtReset.bMin,
                                                                              pLogItem->DateTimeAtReset.bSec);
               RecentResetsLB->Append( wsStr );

            }
          }

        }
      }
    }
  }
}

void WatchDogConnectFrame::OnSocketEvent( wxSocketEvent& event )
{
  // wxString s = _T("OnSocketEvent: ");

  if( WdgConnect != NULL )
  {
    WdgConnect->OnSocketEvent( event );
  }

  /*
  switch(event.GetSocketEvent())
  {
    case wxSOCKET_INPUT:
      // s.Append(_("wxSOCKET_INPUT\n"));
    break;

    case wxSOCKET_LOST:
      // s.Append(_("wxSOCKET_LOST\n"));
    break;

    case wxSOCKET_CONNECTION:
      // s.Append(_("wxSOCKET_CONNECTION\n"));
    break;

    default:
      // s.Append(_("Unexpected event !\n"));
    break;
  }
  */
}

void WatchDogConnectFrame::ConnectWDButtonOnClick( wxCommandEvent& event )
{
  if( WdgConnect != NULL )
  {
    WdgConnect->StopIt();
    WdgConnect->Wait();
    WdgConnect = NULL;
  }

  {

    if( Watchdogs != NULL )
    {
      TWdgConParams WdgConParams;
      u32 dwIdx = WatchDogCB->GetCurrentSelection();

      WdgConParams.dwWDGIdx  = dwIdx;
      WdgConParams.dwPort    = Watchdogs->GetPort( dwIdx );
      WdgConParams.wsAddress = Watchdogs->GetAddress( dwIdx );

      WdgConParams.pWDSocket = pWDSocket;

      WdgConnect = new TWdgConnect( &WdgConParams );
      if( WdgConnect != NULL )
      {
        if( WdgConnect->Create() == wxTHREAD_NO_ERROR )
        {
          WdgConnect->Run();
        }
        else
        {
          wxMessageBox(_T("Error on Creation of WdgConnect Thread"), _T("Thread Error"), wxOK|wxICON_ERROR);
        }
      }
    }
  }
  event.Skip();
}

void WatchDogConnectFrame::DisconnectButtonOnClick( wxCommandEvent& event )
{
  if( WdgConnect != NULL )
  {
    WdgConnect->StopIt();
    WdgConnect->Wait();
    WdgConnect = NULL;
  }

  event.Skip();
}

void WatchDogConnectFrame::SetConnectionStatus( wxString StatusStr )
{
  statusBar->SetStatusText( StatusStr, 2);
}

void WatchDogConnectFrame::GetWDInfoOnClick( wxCommandEvent& event )
{
  SendCmdGetInfo();
  event.Skip();
}

void WatchDogConnectFrame::SendCmdGetInfo()
{
  TWDGProtocol WdgProt;
  if( WdgConnect != NULL )
  {
    GetInfoStatusLabel->SetLabel(_("Requesting info"));
    WdgProt.Header.bCmd = WDG_CMD_GET_INFO;
    WdgConnect->write( &WdgProt );
  }
}

void WatchDogConnectFrame::SetSettingsButtonOnClick( wxCommandEvent& event )
{
  TWDGProtocol WdgProt;
  SetSetSettingsStatus( _("") );

  if( WdgConnect != NULL )
  {
    WdgProt.Header.bCmd = WDG_CMD_SETTINGS;

    WdgProt.Data.WDGSettings.bSettingsCmd = WDGS_SET_SETTINGS;

    WdgProt.Data.WDGSettings.dwSettings      = 0;

    WdgProt.Data.WDGSettings.dwSettings     |= WDG_SET_MASK_RESET_TIME;
    WdgProt.Data.WDGSettings.dwResetTime     = ResetTimeSpin->GetValue();

    WdgProt.Data.WDGSettings.dwSettings     |= WDG_SET_MASK_BEEP;
    WdgProt.Data.WDGSettings.bBeep           = BeeperCB->IsChecked() ? 1 : 0;

    WdgProt.Data.WDGSettings.dwSettings     |= WDG_SET_MASK_TRIGGER_TIME;
    WdgProt.Data.WDGSettings.dwTriggerTime   = WDGTriggerTimeSpin->GetValue();


    if( ResetItCB->IsChecked() )
    {
      WdgProt.Data.WDGSettings.dwSettings     |= WDG_SET_MASK_RESET_IT;
      WdgProt.Data.WDGSettings.bResetIt        = 1;
    }

    WdgProt.Data.WDGSettings.dwSettings     |= WDG_SET_MASK_STOP_RESET;
    WdgProt.Data.WDGSettings.bStopReset      = StopResetsCB->IsChecked() ? 1 : 0;

    if( ClearCntCB->IsChecked() )
    {
      WdgProt.Data.WDGSettings.dwSettings     |= WDG_SET_MASK_CLEAR_CNT;
      WdgProt.Data.WDGSettings.bClearCounters  = 1;
    }

    WdgConnect->write( &WdgProt );
  }
  event.Skip();
}

void WatchDogConnectFrame::SetSetSettingsStatus(wxString StatusStr )
{
  SettingsStatusLabel->SetLabel( StatusStr );
}

void WatchDogConnectFrame::SetGetInfoStatus( wxString StatusStr )
{
  GetInfoStatusLabel->SetLabel( StatusStr );
}

void WatchDogConnectFrame::GetLogButtonOnClick( wxCommandEvent& event )
{
  TWDGProtocol WdgProt;
  wxString wsStr;

  if( (WdgConnect != NULL) && (Watchdogs != NULL) )
  {

    if( !Watchdogs->GettingLogProcess( ) )
    {
      WdgProt.Header.bCmd = WDG_CMD_GET_LOG_ENTRY;

      WdgProt.Data.WDGLogEntry.dwIndex = WDGLogStartEntrySpin->GetValue() - 1;

      Watchdogs->GettingLogProcess( true );

      u32 dwIdx = WatchDogCB->GetCurrentSelection();
      TWDGData * pData = Watchdogs->GetWDGData( dwIdx );
      if( pData != NULL )
      {
        wsStr.Printf( _("getting %2u/%2u entry..."), WDGLogStartEntrySpin->GetValue(), pData->ResetLogEntryCount() );
        SetLogItemsCountText( wsStr );
      }
      WdgConnect->write( &WdgProt );
      GetLogButton->SetLabel(_("Stop it"));
    }
    else
    {
      Watchdogs->GettingLogProcess( false );
    }

  }

  event.Skip();
}

void WatchDogConnectFrame::ClearLogButtonOnClick( wxCommandEvent& event )
{
  TWDGProtocol WdgProt;
  wxString wsStr;

  if( (WdgConnect != NULL) && (Watchdogs != NULL) )
  {

    if( !Watchdogs->GettingLogProcess( ) )
    {
      WdgProt.Header.bCmd = WDG_CMD_CLEAR_LOG;

      WdgConnect->write( &WdgProt );
    }

  }
  event.Skip();
}

