/***************************************************************
 * Name:      WatchdogClass.cpp
 * Purpose:   WatchDog class
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/

#include "types.h"
#include "WatchdogClass.h"
#include "WDGProtocol.h"

#ifdef WDG_TEST_VALUES
static u32 ssdwIndx = 0;
#endif


// ================================================================================

typedef struct WDListItemA {

  u32 dwIndex;
  char * pName;
  char Address[IP_ADDRESS_BUFF_LEN];
  u32 dwPort;

  TWDGData WDGData;



  WDListItemA( u32 dwIdx );
  ~WDListItemA();

} WDListItem;




// ================================================================================


WDListItemA::WDListItemA( u32 dwIdx )
{
  pName       = NULL;
  Address[0]  = 0;
  dwPort      = MAX_U32;
  dwIndex     = dwIdx;

  #ifdef WDG_TEST_VALUES
  char * pPrtNm = "Some Port";
  WDGData.SetValue( GDV_PORT_NAME,         (u32)pPrtNm   );
  WDGData.SetValue( GDV_RUN_TIME,          ssdwIndx + 1  );
  WDGData.SetValue( GDV_RESET_COUNT,       ssdwIndx + 2  );
  WDGData.SetValue( GDV_TRIGGER_COUNT,     ssdwIndx + 3  );
  WDGData.SetValue( GDV_TRRIGGER_TIME,     ssdwIndx + 4  );
  WDGData.SetValue( GDV_LAST_TRIGGER_TIME, ssdwIndx + 5  );
  ssdwIndx++ ;
  #endif
}

WDListItemA::~WDListItemA()
{
  if( pName != NULL )
  {
    delete [] pName;
    pName = NULL;
  }
}

WX_DECLARE_LIST(WDListItem, TWDList);

#include <wx/listimpl.cpp>

WX_DEFINE_LIST( TWDList );

TWDList * pWDList;
// ================================================================================


TWatchdog::TWatchdog()
{
  pWDList     = new TWDList;

  fChanged    = false;
  fNewData    = false;
  fGettingLog = false;
}

TWatchdog::~TWatchdog()
{

  if( pWDList != NULL )
  {
    TWDList::iterator iter;

    for (iter = pWDList->begin(); iter != pWDList->end(); ++iter)
    {
      WDListItem *current = *iter;

      if( current != NULL )
      {
        delete current;
      }
    }

    delete pWDList;
    pWDList = NULL;
  }

}

u32 TWatchdog::Add( char * pSrcName )
{
  u32 deRetVal = MAX_U32;
  WDListItem * pWDListItem;

  if( (pWDList != NULL) && (pSrcName != NULL) )
  {
    u32 dwCount = pWDList->GetCount();

    pWDListItem = new WDListItem( dwCount );
    if( pWDListItem != NULL )
    {
      u32 dwNameLen = strlen( pSrcName );

      char * pName = new char[ dwNameLen + 1];
      if( pName != NULL )
      {
        strncpy( pName, pSrcName, dwNameLen );
        pName[ dwNameLen ] = 0;

        pWDListItem->pName = pName;

        pWDList->Append( pWDListItem );
        deRetVal = dwCount;

        fChanged = true;
      }
      else
      {
        delete pWDListItem;
      }
    }
  }

  return deRetVal;
}

void TWatchdog::SetName( u32 dwWDIdx, char * pSrcName )
{
  if( (pWDList != NULL) && (pSrcName != NULL) )
  {
    TWDList::iterator iter;
    u32 dwIndex = 0;

    if( dwWDIdx < pWDList->GetCount() )
    {
      for (iter = pWDList->begin(); iter != pWDList->end(); ++iter)
      {
        WDListItem *current = *iter;

        if( dwIndex == dwWDIdx )
        {
          if( current != NULL )
          {
            if( current->pName != NULL )
            {
              delete [] current->pName;
            }
            u32 dwNameLen = strlen( pSrcName );

            char * pName = new char[ dwNameLen + 1];
            if( pName != NULL )
            {
              strncpy( pName, pSrcName, dwNameLen );
              pName[ dwNameLen ] = 0;
              current->pName = pName;
              fChanged = true;
            }
          }
          break;
        }
        dwIndex++;
      }
    }
  }

}

char * TWatchdog::GetName( u32 dwWDIdx )
{
  char * pRetVal = NULL;
  if( (pWDList != NULL) )
  {
    TWDList::iterator iter;
    u32 dwIndex = 0;

    if( dwWDIdx < pWDList->GetCount() )
    {
      for (iter = pWDList->begin(); iter != pWDList->end(); ++iter)
      {
        WDListItem *current = *iter;

        if( dwIndex == dwWDIdx )
        {
          if( current != NULL )
          {
            pRetVal = current->pName;
          }
          break;
        }
        dwIndex++;
      }
    }
  }
  return pRetVal;
}

void TWatchdog::SetAddress( u32 dwWDIdx, char * pSrcAddress )
{
  if( (pWDList != NULL) && (pSrcAddress != NULL) )
  {
    TWDList::iterator iter;
    u32 dwIndex = 0;

    if( dwWDIdx < pWDList->GetCount() )
    {
      for (iter = pWDList->begin(); iter != pWDList->end(); ++iter)
      {
        WDListItem *current = *iter;

        if( dwIndex == dwWDIdx )
        {
          if( current != NULL )
          {
            u08 bAddrLen = strlen( pSrcAddress );
            if( bAddrLen >= IP_ADDRESS_BUFF_LEN )
            {
              bAddrLen = IP_ADDRESS_BUFF_LEN - 1;
            }
            strncpy( current->Address, pSrcAddress, IP_ADDRESS_BUFF_LEN );
            current->Address[ bAddrLen ] = 0;
            fChanged = true;
          }
          break;
        }
        dwIndex++;
      }
    }
  }
}

char * TWatchdog::GetAddress( u32 dwWDIdx )
{
  char * pRetVal = NULL;
  if( (pWDList != NULL) )
  {
    TWDList::iterator iter;
    u32 dwIndex = 0;

    if( dwWDIdx < pWDList->GetCount() )
    {
      for (iter = pWDList->begin(); iter != pWDList->end(); ++iter)
      {
        WDListItem *current = *iter;

        if( dwIndex == dwWDIdx )
        {
          if( current != NULL )
          {
            pRetVal = current->Address;
          }
          break;
        }
        dwIndex++;
      }
    }
  }
  return pRetVal;
}

void TWatchdog::SetPort( u32 dwWDIdx, u32 dwSrcPort )
{
  if( (pWDList != NULL) )
  {
    TWDList::iterator iter;
    u32 dwIndex = 0;

    if( dwWDIdx < pWDList->GetCount() )
    {
      for (iter = pWDList->begin(); iter != pWDList->end(); ++iter)
      {
        WDListItem *current = *iter;

        if( dwIndex == dwWDIdx )
        {
          if( current != NULL )
          {
            current->dwPort = dwSrcPort;
            fChanged = true;
          }
          break;
        }
        dwIndex++;
      }
    }
  }
}

u32 TWatchdog::GetPort( u32 dwWDIdx )
{
  u32 deRetVal = MAX_U32;
  if( (pWDList != NULL) )
  {
    TWDList::iterator iter;
    u32 dwIndex = 0;

    if( dwWDIdx < pWDList->GetCount() )
    {
      for (iter = pWDList->begin(); iter != pWDList->end(); ++iter)
      {
        WDListItem *current = *iter;

        if( dwIndex == dwWDIdx )
        {
          if( current != NULL )
          {
            deRetVal = current->dwPort;
          }
          break;
        }
        dwIndex++;
      }
    }
  }
  return deRetVal;
}

void TWatchdog::Delete( u32 dwWDIdx )
{
  if( (pWDList != NULL) )
  {
    TWDList::iterator iter;
    u32 dwIndex = 0;

    if( dwWDIdx < pWDList->GetCount() )
    {
      for (iter = pWDList->begin(); iter != pWDList->end(); ++iter)
      {
        WDListItem *current = *iter;

        if( dwIndex == dwWDIdx )
        {
          if( current != NULL )
          {
            delete current;
          }
          pWDList->DeleteObject( current );
          fChanged = true;
          break;
        }
        dwIndex++;
      }
    }
  }
}

u32 TWatchdog::GetCount()
{
  u32 dwCount = 0;
  if( (pWDList != NULL) )
  {
    dwCount = pWDList->GetCount();
  }
  return dwCount;
}

TWDGData * TWatchdog::GetWDGData( u32 dwWDIdx )
{
  TWDGData * pRetVal = NULL;

  if( (pWDList != NULL) )
  {
    TWDList::iterator iter;
    u32 dwIndex = 0;

    if( dwWDIdx < pWDList->GetCount() )
    {
      for (iter = pWDList->begin(); iter != pWDList->end(); ++iter)
      {
        WDListItem *current = *iter;

        if( dwIndex == dwWDIdx )
        {
          if( current != NULL )
          {
            pRetVal = &(current->WDGData);
          }
          break;
        }
        dwIndex++;
      }
    }
  }
  return pRetVal;
}

