/***************************************************************
 * Name:      WdgConnect.cpp
 * Purpose:   WatchDog connection class
 * Author:    Alexey Govorukhin (MasterAlexei@googlemail.com)
 * Created:   2010
 * Copyright: Alexey Govorukhin (http://www.fun-electronic.net/)
 * License:
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the author nor the names of its contributors may
 *     be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 *
 **************************************************************************/

#ifndef _WDGCONNECT_C
#define _WDGCONNECT_C

#include <wx/fileconf.h>

#include "wx/wx.h"

#include "wx/socket.h"

#include "wx/thread.h"

#include "types.h"
#include "Config.h"

#include "WatchDogConnectMain.h"
#include "WatchdogClass.h"
#include "WDGProtocol.h"
#include "WdgConnect.h"

#include "fcs.h"


#include "GlobalData.h"
#include "Externals.h"



TWdgConnect::TWdgConnect( TWdgConParams * WdgConParams ) : wxThread( wxTHREAD_JOINABLE )
{
  wxIPV4address addr;

  fStopRuning = false;
  bConnected  = false;

  WdgConParam.dwWDGIdx  = WdgConParams->dwWDGIdx;
  WdgConParam.dwPort    = WdgConParams->dwPort;
  WdgConParam.wsAddress = WdgConParams->wsAddress;
  WdgConParam.pWDSocket = WdgConParams->pWDSocket;

  addr.Hostname( WdgConParam.wsAddress );
  addr.Service( WdgConParam.dwPort );

  bRXBuffer = new u08[ sizeof(TWDGProtocol) * 2 ];
  bTXBuffer = new u08[ sizeof(TWDGProtocol) * 2 ];

  memset( (u08 *)&WDGBuffIdx, 0, sizeof(TWDGBuffIdx) );

  if( (bRXBuffer != NULL) && (bTXBuffer != NULL) )
  {
    WdgConParam.pWDSocket->Connect(addr, false);
    WdgConParam.pWDSocket->SetFlags( wxSOCKET_NOWAIT );
  }
}


TWdgConnect::~TWdgConnect()
{
  if( bRXBuffer != NULL )
  {
    delete bRXBuffer ;
  }

  if( bRXBuffer != NULL )
  {
    delete bTXBuffer ;
  }
}


void TWdgConnect::OnSocketEvent( wxSocketEvent& event )
{
  // wxString s = _T("OnSocketEvent: ");

  switch(event.GetSocketEvent())
  {
    case wxSOCKET_INPUT:
    break;

    case wxSOCKET_LOST:
      bConnected = false;
    break;

    case wxSOCKET_CONNECTION:
      memset( (u08 *)&WDGBuffIdx, 0, sizeof(TWDGBuffIdx) );

      // bConnected = true;
      WdgConParam.pWDSocket->SetFlags( wxSOCKET_NOWAIT );
      // MainFrame->SetConnectionStatus(_("Connected"));

    break;

    default:
      // s.Append(_("Unexpected event !\n"));
    break;
  }

}

#define RX_INDEX_IN(idx) ( (WDGBuffIdx.bRxBufferIndex == 0 ? 1 : 0 ) * (sizeof(TWDGProtocol)) + idx )
#define TX_INDEX_OUT(idx) ( (WDGBuffIdx.bTxBufferIndex == 0 ? 1 : 0 ) * (sizeof(TWDGProtocol)) + idx )

#define RX_INDEX(idx) ( WDGBuffIdx.bRxBufferIndex * (sizeof(TWDGProtocol)) + idx )
#define TX_INDEX(idx) ( WDGBuffIdx.bTxBufferIndex * (sizeof(TWDGProtocol)) + idx )


// writes data to internal buffer of WdgConnect class
u32 TWdgConnect::write( TWDGProtocol * pSrc )
{
  u32 dwRetVal = 0;
  TWDGProtocol LocalData;
  u16convert ewCRC;

  memcpy( (u08 *)&LocalData, pSrc, sizeof(TWDGProtocol) - 2 );
  ewCRC.value = CalculateCRC((u08 *)&LocalData, sizeof(TWDGProtocol) - 2 );
  LocalData.dwCRC.value = ewCRC.value;

  memcpy( &bTXBuffer[ TX_INDEX_OUT( 0 ) ], (u08 *)&LocalData, sizeof(TWDGProtocol) );

  if( WDGBuffIdx.fTxData == false )
  {
    WDGBuffIdx.bTxBufferIndex = WDGBuffIdx.bTxBufferIndex == 0 ? 1 : 0;
    dwRetVal = sizeof(TWDGProtocol);
    WDGBuffIdx.bTxIndex = 0;
    WDGBuffIdx.fTxData  = true;
  }

  return dwRetVal;
}

// reads data from internal buffer of WdgConnect class
u32 TWdgConnect::read( TWDGProtocol * pDst )
{
  u32 dwRetVal = 0;

  if( WDGBuffIdx.fRxData )
  {
    memcpy( pDst, &bRXBuffer[ RX_INDEX( 0 ) ],  sizeof(TWDGProtocol) );
    WDGBuffIdx.bRxBufferIndex = WDGBuffIdx.bRxBufferIndex == 1 ? 0 : 1;
    dwRetVal = sizeof(TWDGProtocol);

    WDGBuffIdx.bRxIndex = 0;
    WDGBuffIdx.fRxData = false;
  }

  return dwRetVal;
}



void * TWdgConnect::Entry()
{
  s32 dwBytesToRead  = 0;
  s32 dwBytesToWrite = 0;
  u32 dwConnectcounter = 0;

  MainFrame->SetConnectionStatus(_("Connecting..."));

  while( !TestDestroy() && !fStopRuning )
  {
    if( !bConnected )
    {
      if( WdgConParam.pWDSocket->WaitOnConnect(0, 100) )
      {
        if( WdgConParam.pWDSocket->IsConnected() )
        {
          bConnected = true;
          fStopRuning = false;
          dwConnectcounter = 0;
          MainFrame->SetConnectionStatus(_("Connected"));
        }
        else
        {
          fStopRuning = true;
        }
      }
      else
      {
        dwConnectcounter++;
        if( dwConnectcounter > Config->ConfigData.Options.dwConnectDelay )
        {
          fStopRuning = true;
        }
      }
    }

    if( bConnected && !fStopRuning)
    {
      if( WDGBuffIdx.fTxData && !fStopRuning )                    // Is there some data to send
      {
        dwBytesToWrite = sizeof( TWDGProtocol ) - WDGBuffIdx.bTxIndex;
        if( dwBytesToWrite > 0 )
        {
          WdgConParam.pWDSocket->Write( &bTXBuffer[ TX_INDEX( WDGBuffIdx.bTxIndex ) ], dwBytesToWrite );
          WDGBuffIdx.bTxIndex += WdgConParam.pWDSocket->LastCount();
        }
        else
        {
          WDGBuffIdx.fTxData = false;
        }
      }
      else
      {
        Sleep(1);
      }


      if( ! WDGBuffIdx.fRxData && !fStopRuning )
      {
        dwBytesToRead = sizeof( TWDGProtocol ) - WDGBuffIdx.bRxIndex;                      // Calc count of bytes to read
        if( dwBytesToRead > 0 )                                                             // is there some data to read left
        {
          WdgConParam.pWDSocket->Read( &bRXBuffer[ RX_INDEX( WDGBuffIdx.bRxIndex ) ], dwBytesToRead ); // trye to read it

          WDGBuffIdx.bRxIndex += WdgConParam.pWDSocket->LastCount();                       // cal index of next data transfer
          if( WDGBuffIdx.bRxIndex == sizeof( TWDGProtocol ) )                              // have we read everything
          {
            WDGBuffIdx.fRxData = true;                                                      // Set flag of it
          }
        }
      }
      else
      {
        Sleep(1);
      }
    } // if( bConnected )

    Communication();

  } // while(!TestDestroy())

  if( WdgConParam.pWDSocket->IsConnected() )
  {
    bConnected = false;
    WdgConParam.pWDSocket->Close();
  }

  MainFrame->SetConnectionStatus(_("Disconnected"));

  return NULL;
}

void TWdgConnect::Communication( void )
{
  TWDGProtocol WDGData;
  // volatile int X = 0;

  if( read( &WDGData ) != 0 )
  {
    u16convert ewCRC;
    ewCRC.value = CalculateCRC((u08 *)&WDGData, sizeof(TWDGProtocol) - 2 );
    if( ewCRC.value == WDGData.dwCRC.value )
    {
      switch( WDGData.Header.bCmd )
      {
        case WDG_CMD_GET_INFO:

          if( Watchdogs != NULL )
          {
            TWDGData * pData = Watchdogs->GetWDGData( WdgConParam.dwWDGIdx );
            if( pData != NULL )
            {
              pData->SetValue( GDV_PORT_NAME,         (u32)WDGData.Data.WDGInfo.bCommPort    );
              pData->SetValue( GDV_RUN_TIME,          WDGData.Data.WDGInfo.dwRunTime         );
              pData->SetValue( GDV_RESET_COUNT,       WDGData.Data.WDGInfo.dwResetsCount     );
              pData->SetValue( GDV_TRIGGER_COUNT,     WDGData.Data.WDGInfo.dwTriggerCount    );
              pData->SetValue( GDV_TRRIGGER_TIME,     WDGData.Data.WDGInfo.dwTriggerTime     );
              pData->SetValue( GDV_LAST_TRIGGER_TIME, WDGData.Data.WDGInfo.dwLastTriggerTime );
              pData->SetValue( GDV_RESET_TIME,        WDGData.Data.WDGInfo.dwResetTime       );
              pData->SetValue( GDV_BEEPER,            WDGData.Data.WDGInfo.bBeeping          );
              pData->SetValue( GDV_STOP_RESET,        WDGData.Data.WDGInfo.bStopReset        );
              pData->SetValue( GDV_LOG_ENTRY_COUNT,   WDGData.Data.WDGInfo.dwLogEntryCount   );

              Watchdogs->NewData( true );
            }
          }
          MainFrame->SetGetInfoStatus(_("Info received"));
        break;
        case WDG_CMD_SETTINGS:
          MainFrame->SetSetSettingsStatus( _("Setings are set") );

        break;
        case WDG_CMD_GET_LOG_ENTRY:
        {
          if( (Watchdogs != NULL) )
          {
            TWDGData * pData = Watchdogs->GetWDGData( WdgConParam.dwWDGIdx );
            if( pData != NULL )
            {
              u32 dwLogItemsCount = pData->GetValue( GDV_LOG_ENTRY_COUNT );

              if( (WDGData.Data.WDGLogEntry.dwIndex + 1) == MainFrame->GetWDGLogStartIdx() )  // it is a first requestetd log item
              {                                                                                 // clear list
                pData->ClearLog();
              }

              // add log item to the log list
              pData->AddLogItem( &WDGData.Data.WDGLogEntry.LogItem );

              Watchdogs->NewData( true );

              // request next log item, if any
              if( Watchdogs->GettingLogProcess() && ((WDGData.Data.WDGLogEntry.dwIndex + 1) < dwLogItemsCount) )
              {

                TWDGProtocol WdgProt;
                wxString wsStr;
                if( (WdgConnect != NULL) )
                {
                  WdgProt.Header.bCmd = WDG_CMD_GET_LOG_ENTRY;
                  WdgProt.Data.WDGLogEntry.dwIndex = (WDGData.Data.WDGLogEntry.dwIndex + 1);

                  wsStr.Printf( _("getting %2u/%2u entry..."), WdgProt.Data.WDGLogEntry.dwIndex + 1, dwLogItemsCount );
                  MainFrame->SetLogItemsCountText( wsStr );

                  WdgConnect->write( &WdgProt );

                }
              }
              else
              {
                Watchdogs->GettingLogProcess( false );
              }
            }
          }
        }
        break;

        case WDG_CMD_CLEAR_LOG:
          if( (Watchdogs != NULL) )
          {
            TWDGData * pData = Watchdogs->GetWDGData( WdgConParam.dwWDGIdx );
            if( pData != NULL )
            {
              pData->ClearLog();
              Watchdogs->NewData( true );
            }
          }
        break;

        default:
        break;
      }
    }
  }
  else
  {
    Sleep(1);
  }

}

#endif

